﻿<#
 - This script is intended to collect information about your Passwordstate environment, and the server that hosts your Passwordstate web site.  This will not change any settings on your server.
 - To run this script, ensure you are running Powershell as an Administrator
 - When the script finishes running, please email the serverinfo.txt or serverinfo.zip file to Click Studios on support@clickstudios.com.au

Updated 2nd May 2020
    - Make a copy of web.config file to temp location, and decrypt it if required
    - Delete backup web.config file from Temp location
    - Query Local Administrators Group
    - Updated NSLookup to work with Bindings with no Hosts name set
    - IIS Authentication now queried for Anonymous and Windows authentiction types on Passwordstate website
    - Added more SQL Queries to gather information from database

Updated 15th May 2020
    - Removed a re-encryption of the web.config file that was no longer necessary
    - Changed Local Administrators query to accomodate languages other than English
    - Changed the decryption process for the web.config file to accomodate languages other than English

Updated 27th May 2020
    - Added open port test to domain on port 389 and 636

Updated 25th June 2020
    - Added queries to find the start and stop times of the Passwordstate Windows Service

Updated 26th June 2020
    - Added query to find current time on both web server, and database server

Updated 17th July 2020
    - Added registry check to find where Passwordstate service is pointing to

Updated 19th September 2020
    - Added SQL query to return Active Directory domains
    - Changed option to save file to a custom folder - We now just save it directly to the Script Root folder
    - Commented out the AD Port checks - Test-NetConnection does not work in older versions of Powershell.

Updated 16th November 2020
    - Fix sorting of Application Event Logs to show most recent first

Updated 18th January 2021
    - Added signature to script
    - Removed open port Test for ldaps

Updated 19th January 2021
	- Added new code to query the Passive Node status.  Version 8 will query the web.config file, and Passwordstate 9 will query the database

Updated 6th September 2021
    - Added Powershell session error capturing
    - Fixed Add Remove Programs
    - Added extra Event Log capturing
    - Added web.config file to zip file
    - Improved DNS lookup for all URLs
    - Added Trace Route for all URLs
    - Added better formatting for easier reading
    - Added open port test to domain

Updated 12th October 2021
    - Removed Versioning from Add/Remove Programs

Updated 25th November 2021
    - Updated System Settings SQL query to not refernce obsolete fields

Updated 7th February 2022
    - Removed Forms based database query
    - Removed domain port check

Updated 23rd March 2022
    - Updated SQL query for BackupSettings table

Updated 4th May 2022
    - Updated SQL query for Authorised Web Servers

Updated 12th May 2022
    - Added database size SQL query
    - Added .NET Framework versioning
    - Split output into separate log files
    - Removed one Memory Test

Updated 7th July 2022
    - Replaced database SQL Query to give more accurate results
    - Removed 'else' statement of a foreach loop that should not have been there
    - Removed the If statement for the SELECT * FROM WebServers SQL script
    - Added Installed Programs back
    - Added collecting custom entries from Hosts file
    - Fixed the domain check
    - Fixed error capturing for local intranet sites
    - If using an MSA account, we now write an empty Error Console log file

Updated 19th September 2022
    - Fixed issue with Active Directory domains SQL query
    - Added extra log file for Email notifications database queries
    - Added new query to get the oldest auditing record
    - Cleaned up some headers to display better

Updated 16th February 2023
    - Added SQL Query to find Browser Extension settings from Administratrion area
    - Added check for location of Passwordstate Windows service as it was moved in build 9708
    - Removed permission check for Passwordstate.exe as it was never used

Updated 1st March 2023 - Version 1.22
    - Fixed an issue with the Passwordstate.exe file version check
    - Added versioning to log file output

Updated 22nd March 2023 - Version 1.23
    - Add commands to list all Powershell Modules installed

Updated 29th May 2023 - Version 1.24
    - Added information about Authentication Options in System Settings and User Account Policies
    - Changed the way we obtain data from web.config file to be more accurate, in the event the connection string is not in a standard format
    - Changed the redaction of the web.config file to only redact the password and secret keys
    - Added total database size
    - Removed High Availability DB permission check
    - Moved Ignored URLs into a dedicated log file
    - Added query for TLS values out of registry
    - Added additional information about blocked Email Notifications
    - Added query to detect any queued emails
    - Added NSLookup of each domain
  
Updated 20th February 2024 - Version 1.25
    - Added upgrade dates and versions
    - Added back SQL Database tables size
    - Checked if Allowed IP Ranges was configured
    - Added NSLookup and Trace Route added for Base URL

Updating 4th March 2024 - Version 1.26
    - cleaned up Active Directory Domains to display better
    - Fixed and issue with Authentication options database query
    - Browser Extensions URL from system settings
    - Adjusted Email Notification queries to make easier to intepret
    - Cleaned up System Settings database queries, to display better
    - Changes SQL Services query to look for all SQL Services on webserver, if they exist
#>

#########################################################################################################
# Clear Shell and Variables
#########################################################################################################
cls
Write-Host "Script has started"


#########################################################################################################
# Perform a check to ensure Powershell is running as Administrator
#########################################################################################################
If (-NOT ([Security.Principal.WindowsPrincipal][Security.Principal.WindowsIdentity]::GetCurrent()).IsInRole([Security.Principal.WindowsBuiltInRole] "Administrator"))
{
	$wshell = New-Object -ComObject Wscript.Shell
	$wshell.Popup("This script requires your Powershell session to be elevated.`n`nPlease run again in a new shell that is run as Administrator", 0, "Warning", 0x0)
	Break
}

#########################################################################################################
# Set variables to be used later in script and import modules
#########################################################################################################
$ScriptVersion = "1.26"
$Path = $PSScriptRoot
$Log1 = "$Path\1 - WebServer Resources.txt"
$Log2 = "$Path\2 - IIS and DNS Information.txt"
$Log3 = "$Path\3 - Install Information.txt"
$Log4 = "$Path\4 - Database SQL Queries.txt"
$Log5 = "$Path\5 - Error Console.txt"
$Log6 = "$Path\6 - Event Logs.txt"
$Log7 = "$Path\7 - Email Notifications.txt"
$Log8 = "$Path\8 - Authentication Options.txt"
$Log9 = "$Path\9 - Ignored URLs.txt"
$Log10 = "$Path\10 - UpgradeLog.txt"
$Log11 = "$Path\11 - Table Sizes.txt"

$error.Clear()
$ErrorActionPreference = "SilentlyContinue" 
Import-Module WebAdministration

#########################################################################################################
# Remove the serverinfo.zip file if it exists on disk
#########################################################################################################
Write-Host "Removing existing serverinfo.zip file if it exists on disk"
$File = Test-Path "$Path\serverinfo.zip"
If ($File -eq $true) { Remove-item "$Path\serverinfo.zip" -force }

#########################################################################################################
# Get the start time of the script
#########################################################################################################
Write-Host "Getting the start time of the script"
$Start = get-date
"Script begin time is $Start" | Out-File $Log1
"Script Version: $ScriptVersion" | Out-File $Log1 -Append
"" | Out-File $Log1 -append
"" | Out-File $Log1 -append


#########################################################################################################
# Query registry to find all applications installed on this server
#########################################################################################################
Write-Host "Listing all programs installed on server"
"Contents of Add/Remove Progams" | Out-File $Log1 -append
Get-ChildItem HKLM:\Software\Microsoft\Windows\CurrentVersion\Uninstall | 
% {Get-ItemProperty $_.PsPath} | 
where {$_.Displayname -and ($_.Displayname -match ".*")} | 
sort Displayname  | select DisplayName, Publisher -Unique | Format-Table | Out-File $Log1 -append
"" | Out-File $Log1 -append
"" | Out-File $Log1 -append

#########################################################################################################
# Report back the name of the web server
#########################################################################################################
Write-Host "Getting the name of the server"
"Web Server Name" | Out-File $Log1 -append
$Servername = (gwmi win32_computersystem).DNSHostName
$Servername | Out-File $Log1 -append
"" | Out-File $Log1 -append
"" | Out-File $Log1 -append

#########################################################################################################
# Check if the server is part of a domain
#########################################################################################################
Write-Host "Checking is server is part of a domain"
"Domain Check" | Out-File $Log1 -append
$DomainCheck = (Get-WmiObject -Class Win32_ComputerSystem).PartOfDomain
if ($DomainCheck -eq $true) 
{
$domain = (gwmi win32_computersystem).Domain
"$servername is part of the $domain domain" | Out-File $Log1 -append } 
else 
{"$servername is not joined to a domain" | Out-File $Log1 -append }
"" | Out-File $Log1 -append
"" | Out-File $Log1 -append

#########################################################################################################
# Get the current time of the web server
#########################################################################################################
Write-Host "Getting the current time of the server"
[string]$time = get-date 
"Current Time of Web Server" | Out-File $Log1 -append
$time | Out-File $Log1 -append
"" | Out-File $Log1 -append
"" | Out-File $Log1 -append

#########################################################################################################
# Check last time server was booted up
#########################################################################################################
Write-Host "Finding out last boot up time"
"Last boot time of $Servername" | Out-File $Log1 -append
[string](Get-CimInstance -ClassName win32_operatingsystem).lastbootuptime | Out-File $Log1 -append
"" | Out-File $Log1 -append
"" | Out-File $Log1 -append

#########################################################################################################
# Get the language the operating system is set to
#########################################################################################################
Write-Host "Querying what language the server is using"
"Operating System Language" | Out-File $Log1 -append
[string](Get-Culture).DisplayName | ft -AutoSize | Out-File $Log1 -append
"" | Out-File $Log1 -append
"" | Out-File $Log1 -append

#########################################################################################################
# Get the Operating System Version
#########################################################################################################
Write-Host "Querying what version the operating system is"
"Operating System Version" | Out-File $Log1 -append
(Get-WmiObject Win32_OperatingSystem).Caption | Out-File $Log1 -append
"" | Out-File $Log1 -append
"" | Out-File $Log1 -append

#########################################################################################################
# Get IPV4 Address of server
#########################################################################################################
Write-Host "Get IPV4 Address of server"
"IPV4 Address" | Out-File $Log1 -append
$ipV4 = (Test-Connection -ComputerName (hostname) -Count 1  | Select -ExpandProperty IPV4Address).IPAddressToString 
$ipV4 | ft -AutoSize | Out-File $Log1 -append
"" | Out-File $Log1 -append
"" | Out-File $Log1 -append

#########################################################################################################
# Get information about Powershell version
#########################################################################################################
Write-Host "Querying the version of Powershell installed"
"Powershell Version" | Out-File $Log1 -append
($host.version).Major | Out-File $Log1 -append
"" | Out-File $Log1 -append
"" | Out-File $Log1 -append

#########################################################################################################
# Get installed Powershell modules
#########################################################################################################
Write-Host "Getting all Powershell install modules"
"Powershell Module Check" | Out-File $Log1 -append
$PSModule = Get-Module WebAdministration
if ($PSModule.name -eq 'WebAdministration')
{
"The Powershell WebAdministration module is installed on this server" | Out-File $Log1 -append
}
else
{
"The Powershell WebAdministration module is not installed on this server" | Out-File $Log1 -append
}
"" | Out-File $Log1 -append
"" | Out-File $Log1 -append

#########################################################################################################
# Check disk space on server
#########################################################################################################
Write-Host "Calculating free disk space"
"Free Disk Space" | Out-File $Log1 -append
Get-WmiObject -Class Win32_logicaldisk -Filter "DriveType = '3'" | Select-Object -Property DeviceID, @{L='FreeSpaceGB';E={"{0:N2}" -f ($_.FreeSpace /1GB)}}, @{L="Capacity";E={"{0:N2}" -f ($_.Size/1GB)}} | Out-File $Log1 -append
"" | Out-File $Log1 -append
"" | Out-File $Log1 -append

#########################################################################################################
# Get detailed IP Addresses of server
#########################################################################################################
Write-Host "Detailed IP Address information on server"
"Detailed IP Address information on server $Servername" | Out-File $Log1 -append
ipconfig /all | Out-File $Log1 -append
"" | Out-File $Log1 -append
"" | Out-File $Log1 -append

#########################################################################################################
# .Net Framework Version
#########################################################################################################
Write-Host "Checking .NET Framework version"
$NETRelease = (Get-ItemProperty -Path 'HKLM:\SOFTWARE\Microsoft\NET Framework Setup\NDP\v4\Full\').Release
$NETVersion = (Get-ItemProperty -Path 'HKLM:\SOFTWARE\Microsoft\NET Framework Setup\NDP\v4\Full\').Version
".NET Framework Version" | Out-File $Log1 -append
$NETVersion | Out-File $Log1 -append
"" | Out-File $Log1 -append
".NET Framework Release Build" | Out-File $Log1 -append
$NETRelease | Out-File $Log1 -append
"" | Out-File $Log1 -append
"" | Out-File $Log1 -append

#########################################################################################################
# Test which displays amount of free memory
#########################################################################################################
Write-Host "Test which displays amount of free memory"
"Free Memory on $servername" | Out-File $Log1 -append
Function Test-MemoryUsage {
[cmdletbinding()]
Param()
$os = Get-Ciminstance Win32_OperatingSystem
$pctFree = [math]::Round(($os.FreePhysicalMemory/$os.TotalVisibleMemorySize)*100,2)
if ($pctFree -ge 45) {
$Status = "OK"
}
elseif ($pctFree -ge 15 ) {
$Status = "Warning"
}
else {
$Status = "Critical"
}
$os | Select @{Name = "Status";Expression = {$Status}},
@{Name = "PctFree"; Expression = {$pctFree}},
@{Name = "FreeGB";Expression = {[math]::Round($_.FreePhysicalMemory/1mb,2)}},
@{Name = "TotalGB";Expression = {[int]($_.TotalVisibleMemorySize/1mb)}}
}

Test-MemoryUsage | Out-File $Log1 -append
"" | Out-File $Log1 -append


#########################################################################################################
# Detailed system memory usage for each process running 
#########################################################################################################
Write-Host "Getting Detailed Information about Server Memory for each running process"
"Getting Detailed Memory Statistics" | Out-File $Log1 -append
$Computername = $env:computername
get-process -computername $Computername | Group-Object -Property ProcessName | Format-Table Name, @{n='Mem (KB)';e={'{0:N0}' -f (($_.Group|Measure-Object WorkingSet -Sum).Sum / 1KB)};a='right'} -AutoSize | Out-File $Log1 -append
"" | Out-File $Log1 -append
"" | Out-File $Log1 -append


"#########################################################################################################" | Out-File $Log2 -append
"# IIS & DNS Section" | Out-File $Log2 -append
"#########################################################################################################" | Out-File $Log2 -append
"" | Out-File $Log2 -append

#########################################################################################################
# List all websites on Server
#########################################################################################################
Write-Host "List all websites on Server"
"List of All Websites in IIS" | Out-File $Log2 -append
(Get-ChildItem -Path IIS:\Sites).Name | Out-File $Log2 -append
"" | Out-File $Log2 -append
"" | Out-File $Log2 -append

#########################################################################################################
# Check we have a website called Passwordstate on this server
#########################################################################################################
Write-Host "Check to confirm passwordstate website exists on server"
$website = Get-WebSite -Name passwordstate
if ($website -eq $null){
"Check to Confirm Passwordstate Website Exists on Server" | Out-File $Log2 -append
"Error: We were unable to find a website called Passwordstate on this server. We will now exit this script." | Out-File $Log2 -append
Write-Host "Getting the end time of the script"
# Get the end time of the script
$End = get-date
"Script finish time is $End" | Out-File $Log2 -append
"" | Out-File $Log2 -append
"" | Out-File $Log2 -append

Write-Host "Testing whether the compress-archive commandlet is available in Powershell"
if ($host.version.Major -ge 5)
{
Write-Host "Compressing the text file"
# Next we will compress the txt file to a zip file
"Currently Compressing the output file - The script has now ended" | Out-File $Log2 -append
Compress-Archive -literalpath $Path\ServerInfo.txt -CompressionLevel Optimal -DestinationPath $Log1 -Force
Remove-Item -path $Path\ServerInfo.txt -force

#Add pop up to confirm script has complete
$wshell = New-Object -ComObject Wscript.Shell
$wshell.Popup("Script is complete. Please email $Log1 to support@clickstudios.com.au",0,"Process Complete",0x0) | Out-Null
}
else
{
#Add pop up to confirm script has complete
$wshell = New-Object -ComObject Wscript.Shell
$wshell.Popup("Script is complete. Please email $Path\serverinfo.txt to support@clickstudios.com.au. 

You may need to use a program to manually compress this file if it is too large to send via email.",0,"Process Complete",0x0) | Out-Null
}
Break
}
else
{
"Check to confirm passwordstate website exists on server" | Out-File $Log2 -append
"Passwordstate website exists on this server" | Out-File $Log2 -append
"" | Out-File $Log2 -append

}

#########################################################################################################
# Query All IIS Bindings
#########################################################################################################
Write-Host "Querying the web bindings for the Passwordstate website"
"Passwordstate IIS Bindings" | Out-File $Log2 -append
Get-webbinding -Name passwordstate | Where-Object { $_.protocol -eq 'https' } | Select Protocol, BindingInformation | ft -AutoSize | Out-File $Log2 -append

#########################################################################################################
# Perform an NSLookup of URLs
#########################################################################################################
Write-Host "Perform an nslookup of the https bindings"
"NSLookup of Passwordstate URL/s (From IIS Bindings)" | Out-File $Log2 -append
"" | Out-File $Log2 -append
$urls = Get-webbinding -Name Passwordstate | Where-Object { $_.protocol -eq 'https' }

foreach($url in $urls)
{
    [string]$a = $url.bindinginformation
    ($url = $a.split(":")[-1]) > $null 2>&1
    if ($url -eq '' -or $url -eq $null)
        {
        "Warning: Binding is empty so no NSLookup required" | Out-File $Log2 -append
         "" | Out-File $Log2 -append
         "" | Out-File $Log2 -append
        }
        else
        {
        "Attempting to perform a NSLookup of $url" | Out-File $Log2 -append
        $DNS  = Resolve-DnsName -Name $url
        $ServerDNS = $DNS.Name
        $ServerDNS  = ($ServerDNS).replace("$url","")
        $DNSIP  = $DNS.IP4Address
            if ($DNS -eq '' -or $DNS -eq $null)
                {
                "Error: $url could not be resolved - Check DNS is set correctly for this URL"| Out-File $Log2 -append
                }
            else
                {
                "Server Name: $ServerDNS" | Out-File $Log2 -append
                "IP Address: $DNSIP" | Out-File $Log2 -append
                }
            "" | Out-File $Log2 -append
            "" | Out-File $Log2 -append
        }
}

#########################################################################################################
# Perform an Trace Route of URLs
#########################################################################################################
Write-Host "Perform an nslookup of the https bindings"
"Trace Route of Passwordstate URL/s (From IIS Bindings)" | Out-File $Log2 -append
"" | Out-File $Log2 -append
$urls = Get-webbinding -Name Passwordstate | Where-Object { $_.protocol -eq 'https' }

foreach($url in $urls)
{
    [string]$a = $url.bindinginformation
    ($url = $a.split(":")[-1]) > $null 2>&1
    if ($url -eq '' -or $url -eq $null)
        {
        "Warning: Binding is empty so no Trace Route required" | Out-File $Log2 -append
         "" | Out-File $Log2 -append
         "" | Out-File $Log2 -append
        }
        else
        {
        "Attempting to perform a Trace Route to $url" | Out-File $Log2 -append
        Tracert $url "" | Out-File $Log2 -append
            "" | Out-File $Log2 -append
            "" | Out-File $Log2 -append
        }
}

#########################################################################################################
# Get information from IIS about Application pools
#########################################################################################################
Write-Host "Finding out information about the Application Pools in IIS"
if ($website -ne $null){ 
"" | Out-File $Log2 -append
"Look for all Application Pools and find their state, and identity" | Out-File $Log2 -append
Get-WebApplication -Site passwordstate | ft -AutoSize | Out-File $Log2 -append
Get-ChildItem -Path IIS:\AppPools\ | Select-Object name, state, @{ e = { $_.processModel.username }; l = "username" }, @{ e = { $_.processModel.identityType }; l = "identityType" } | ft -AutoSize | Out-File $Log2 -append
"" | Out-File $Log2 -append
}
else
{
"Website is not named as the standard Passwordstate name, so we are not checking Application Pools. " | Out-File $Log2 -append
"" | Out-File $Log2 -append
"" | Out-File $Log2 -append
}

#########################################################################################################
# Check IIS Authentication for the Passwordstate website
#########################################################################################################
Write-Host "Check IIS Authentication for the Passwordstate website"
"IIS Authentication Status" | Out-File $Log2 -append
$AnonAuth = Get-WebConfiguration system.webServer/security/authentication/anonymousAuthentication -PSPath IIS:\ -Location passwordstate
$WindowsAuth = Get-WebConfiguration system.webServer/security/authentication/windowsAuthentication -PSPath IIS:\ -Location passwordstate
$AnonEnabled = $AnonAuth.enabled
$WinEnabled = $WindowsAuth.enabled

if ($AnonEnabled -eq 'True')
    {
    "Anonymous Authentication Status: Enabled" | Out-File $Log2 -append
    }
else
    {
    "Anonymous Authentication Status: Disabled" | Out-File $Log2 -append
    }

if ($WinEnabled -eq 'True')
    {
    "Windows Authentication Status: Enabled" | Out-File $Log2 -append
    }
else
    {
    "Windows Authentication Status: Disabled" | Out-File $Log2 -append
    }

"" | Out-File $Log2 -append
"" | Out-File $Log2 -append


#########################################################################################################
# Query local intranet zones
#########################################################################################################
Write-Host "Querying the local intranet zones"
"Local Intranet Zone URLs" | Out-File $Log2 -append
function Get-SiteToZoneAssignment{
    <#
    .Synopsis
    Returns Internet Explorer Site to Zone assignments.
 
    .Description
    Returns a list of sites in the Trusted, Intranet or Restricted Sites of Internet explorer which are defined through Group Policy.
 
    Important: User defined are not returned.
 
    .Example
    # Displays all sites
    Get-SiteToZoneAssignment
    #>
    [CmdletBinding()]
    param()
    $_RegKeyList1 = @()
    $_RegKeyList2 = @()
    $_RegKeyInfo  = @()
    Write-Verbose "Loading registry key 'HKCU:\SOFTWARE\Policies\Microsoft\Windows\CurrentVersion\Internet Settings\ZoneMapKey'"
    $_RegKeyList1 = $(Get-Item 'HKCU:\SOFTWARE\Policies\Microsoft\Windows\CurrentVersion\Internet Settings\ZoneMapKey' -ErrorAction SilentlyContinue)
    Write-Verbose "Loading registry key 'HKLM:\SOFTWARE\Policies\Microsoft\Windows\CurrentVersion\Internet Settings\ZoneMapKey'"
    $_RegKeyList2 = $(Get-Item 'HKLM:\SOFTWARE\Policies\Microsoft\Windows\CurrentVersion\Internet Settings\ZoneMapKey' -ErrorAction SilentlyContinue)
    
    
    Write-Information "Found $($_RegKeyList1.Count) Site to zone assignments for Current User."
    Foreach($_RegValue in $_RegKeyList1.Property){
         $Value = ($_RegKeyList1 | Get-ItemProperty).$($_RegValue)
         Switch($Value){
              0 {$_ZoneType = 'My Computer'}
              1 {$_ZoneType = 'Local Intranet Zone'}
              2 {$_ZoneType = 'Trusted sites Zone'}
              3 {$_ZoneType = 'Internet Zone'}
              4 {$_ZoneType = 'Restricted Sites Zonet'}
              Default { break }
         }
         $newEntry = New-Object -TypeName PSObject
         Add-Member -InputObject $newEntry -MemberType NoteProperty -Name "Zone" -Value $_ZoneType
         Add-Member -InputObject $newEntry -MemberType NoteProperty -Name "Url" -Value $_RegValue
         Write-Verbose "Detected '$($newEntry.Url)' --> '$($newEntry.Zone)'"
         $_RegKeyInfo += $newEntry

    }
    Write-Information "Found $($_RegKeyList2.Count) Site to zone assignments for Current Machine."
    Foreach($_RegValue in $_RegKeyList2.Property){
         $Value = ($_RegKeyList2 | Get-ItemProperty).$($_RegValue)
         Switch($Value){
              0 {$_ZoneType = 'My Computer'}
              1 {$_ZoneType = 'Local Intranet Zone'}
              2 {$_ZoneType = 'Trusted sites Zone'}
              3 {$_ZoneType = 'Internet Zone'}
              4 {$_ZoneType = 'Restricted Sites Zonet'}
              Default { break }
         }
         $newEntry = New-Object -TypeName PSObject
         Add-Member -InputObject $newEntry -MemberType NoteProperty -Name "Zone" -Value $_ZoneType
         Add-Member -InputObject $newEntry -MemberType NoteProperty -Name "Url" -Value $_RegValue
         Write-Verbose "Detected '$($newEntry.Url)' --> '$($newEntry.Zone)'"
         $_RegKeyInfo += $newEntry

    }
    return $_RegKeyInfo
}

Get-SiteToZoneAssignment | Out-File $Log2 -append

"" | Out-File $Log2 -append


#########################################################################################################
# Get custom entries out of Hosts file
#########################################################################################################
Write-Host "Gathering custom DNS entries from Hosts file"
"Information from C:\Windows\System32\drivers\etc\hosts file" | Out-File $Log2 -append
$hostfile = (get-content C:\Windows\System32\drivers\etc\hosts | Where { $_ -notmatch "^#" }).Trim() -ne ''
if ($hostfile -ne "")
{
$hostfile | Out-File $Log2 -append
"" | Out-File $Log2 -append
"" | Out-File $Log2 -append
}
else
{
"Hosts file contains no custom entries. " | Out-File $Log2 -append
"" | Out-File $Log2 -append
"" | Out-File $Log2 -append
}


"#########################################################################################################" | Out-File $Log3 -append
"# Install Information and Upgrade Logs" | Out-File $Log3 -append
"#########################################################################################################" | Out-File $Log3 -append
"" | Out-File $Log3 -append

#########################################################################################################
# List all Files and Folders in Install Directory
#########################################################################################################
Write-Host "Listing the Folders and Files in the Passwordstate directory"
"Passwordstate Install Directory Properties" | Out-File $Log3 -append
$Site = Get-WebSite -Name passwordstate
$InstallPath = $Site.PhysicalPath
Get-ChildItem -Path $Installpath | select fullname | Out-File $Log3 -append
"" | Out-File $Log3 -append
"" | Out-File $Log3 -append


#########################################################################################################
# Get Files and Folders from Gateway Install Directory
#########################################################################################################
Write-Host "Passwordstate Gateway Directory Properties"
"Passwordstate Gateway Directory Properties" | Out-File $Log3 -append
$Site = Get-WebSite -Name Passwordstate
$InstallPath = $Site.PhysicalPath
$HostsPath = "$InstallPath/Hosts/Gateway"
Get-ChildItem -Path $Hostspath | select fullname | Out-File $Log3 -append
"" | Out-File $Log3 -append
"" | Out-File $Log3 -append

#########################################################################################################
# Get Upgrade versions and dates
#########################################################################################################
Write-Host "Retrieving upgrade dates and versions"
"Retrieving upgrade dates and versions" | Out-File $Log10 -append
$UpgradeDates = Get-ItemProperty hklm:\SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\* | select-object DisplayName, InstallDate, InstallLocation, DisplayVersion | where-object DisplayName -EQ "passwordstate" | sort-object InstallDate -Descending | Out-File $Log10 -append
"" | Out-File $Log10 -append
"" | Out-File $Log10 -append

#########################################################################################################
# List Contents of Upgrade Log File
#########################################################################################################
Write-Host "Getting the content of the Upgrade log if there is any"
"Upgrade Log Contents" | Out-File $Log10 -append
$UpgradeLog = "$InstallPath\Upgrades\Upgradelog.txt"
$UpgradeLogResults = test-path $Upgradelog

    If ($UpgradelogResults -eq $true)
    {
    "Upgrade Log Contents" | Out-File $Log10 -append
    Get-Content $Upgradelog | Out-File $Log10 -append
    }
    else
    {
    "Upgrade Log file was not found indicating last upgrade was successful" | Out-File $Log10 -append
    }
    "" | Out-File $Log10 -append
    "" | Out-File $Log10 -append


#########################################################################################################
# Query for Local Administrators
#########################################################################################################
Write-Host "Local Administrators"
"Local Administrators" | Out-File $Log3 -append
$admins_group = ([System.Security.Principal.SecurityIdentifier]'S-1-5-32-544').Translate( [System.Security.Principal.NTAccount]).Value 
$parts = $admins_group -split '\\' 
$groupname = $parts[-1] 
$Admins = Get-LocalGroupMember -Group $groupname

Foreach ($Admin in $Admins)
{
"$Admin" | Out-File $Log3 -append
}
"" | Out-File $Log3 -append
"" | Out-File $Log3 -append

#########################################################################################################
# Get Information about Certificates installed on Server
#########################################################################################################
Write-Host "Listing all certificates"
"Server Certificate information for each IIS binding" | Out-File $Log3 -append

$Sites = Get-ChildItem -Path IIS:SSLBindings | Where-Object {$_.Sites.Value -eq "passwordstate"}
ForEach ($Binding in $Sites)
{  
$certificate = Get-ChildItem -Path CERT:LocalMachine/My | Where-Object -Property Thumbprint -EQ -Value $Binding.Thumbprint
$FriendlyName = $certificate.FriendlyName
$DNSNAme = $certificate.DnsNameList
$ExpiryDate = $certificate.NotAfter
$Issuer = $certificate.Issuer
$Divider = ":"

$HostName = $Binding.Host
if ($HostName -eq '' -or $HostName -eq $null){$HostName = "<No Host Name Set>"}

$Port = $Binding.Port
if ($Port -eq '443')
{$Port = ''
$Divider = ''
}

"" | Out-File $Log3 -append
"Binding:           " + "https://" + "$HostName" + "$Divider" + "$Port" | Out-File $Log3 -append
"Friendly Name:     $FriendlyName" | Out-File $Log3 -append
"DNS NAme:          $DNSNAme" | Out-File $Log3 -append
"Expiry Date:       $ExpiryDate" | Out-File $Log3 -append
"Issuer:            $Issuer" | Out-File $Log3 -append
}

"" | Out-File $Log3 -append
"" | Out-File $Log3 -append

#########################################################################################################
# Get information about Passwordstate Service
#########################################################################################################
Write-Host "Finding out information about the Passwordstate service"
$status = (Get-Service "passwordstate service").Status
$pwsuser = (Get-WmiObject Win32_Service -Filter "Name='passwordstate Service'").StartName
"Passwordstate Service Information" | Out-File $Log3 -append
"Current State: $status" | Out-File $Log3 -append
"Log On As User: $pwsuser" | Out-File $Log3 -append
"" | Out-File $Log3 -append
"" | Out-File $Log3 -append

#########################################################################################################
# Get registry information about Passwordstate Service
#########################################################################################################
Write-Host "Finding out registry information about the Passwordstate service"
cd hklm:
$key = (Get-ItemProperty -Path "HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\Services\Passwordstate Service" -Name "ImagePath").ImagePath
"Registry Information about Passwordstate Service - Path to Service in Registry:" | Out-File $Log3 -append
"Path: $key" | Out-File $Log3 -append
"" | Out-File $Log3 -append
"" | Out-File $Log3 -append

#########################################################################################################
# Get information about Passwordstate web files version
#########################################################################################################
Write-Host "Finding out what version of Passwordstate web files are installed"
$fileversion = [System.Diagnostics.FileVersionInfo]::GetVersionInfo("$key").FileVersion
"Passwordstate Executable Information" | Out-File $Log3 -append
"Version: $fileversion" | Out-File $Log3 -append
"" | Out-File $Log3 -append
"" | Out-File $Log3 -append

#########################################################################################################
# Get more information about Passwordstate Service
#########################################################################################################
Write-Host "Finding out information about when Passwordstate Service was started, and stopped"
$Started = (Get-EventLog -LogName "System" -Source "Service Control Manager" -EntryType "Information" -Message "*Passwordstate Service*running*" -Newest 25).TimeGenerated
$Stopped = (Get-EventLog -LogName "System" -Source "Service Control Manager" -EntryType "Information" -Message "*Passwordstate Service*stopped*" -Newest 25).TimeGenerated
"Last 25 Times Passwordstate Service was Started" | Out-File $Log3 -append
$Started | Out-File $Log3 -append
"" | Out-File $Log3 -append
"Last 25 Times Passwordstate Service was Stopped" | Out-File $Log3 -append
$Stopped | Out-File $Log3 -append
"" | Out-File $Log3 -append
"" | Out-File $Log3 -append


#########################################################################################################
# Get information about Microsoft SQL Service
#########################################################################################################
Write-Host "Finding out information about the Microsoft SQL services if they exist"
$SQLServices = Get-WmiObject -Class Win32_Service -Filter "Name LIKE'SQL%'"
"Microsoft SQL Service Information" | Out-File $Log3 -append
"" | Out-File $Log3 -append

If ($SQLServices -eq $null)
{
	Write-Output "The Microsoft SQL service is not installed on this computer" | Out-File $Log3 -append
	"" | Out-File $Log3 -append
}
Else
{
    foreach ($Service in $SQLServices)
    {
    $ServiceName = $Service.Name
    $ServiceStatus = (Get-Service $ServiceName).Status
	

    $LogonUser = (Get-WmiObject Win32_Service -Filter "Name='$ServiceName'").StartName
    $DisplayName = (Get-WmiObject Win32_Service -Filter "Name='$ServiceName'").DisplayName
	
    "Display name: $DisplayName" | Out-File $Log3 -append
	"Current State: $ServiceStatus" | Out-File $Log3 -append
    "Log On As User: $LogonUser" | Out-File $Log3 -append
	"" | Out-File $Log3 -append
    }
}
"" | Out-File $Log3 -append
"" | Out-File $Log3 -append

#########################################################################################################
# Get information about Passwordstate Gateway Service
#########################################################################################################
Write-Host "Querying Passwordstate gateway Service information"
$PWSGate = Get-WmiObject -Class Win32_Service -Filter "Name='passwordstate-gateway'"

If ($PWSGate -eq $null)
{
    "Passwordstate-Gateway Service Information" | Out-File $Log3 -append
	Write-Output "The Passwordstate-Gateway service is not installed on this computer" | Out-File $Log3 -append
	"" | Out-File $Log3 -append
    "" | Out-File $Log3 -append
}
Else
{
    $status1 = (Get-Service "passwordstate-gateway").Status
	$pwsuser1 = (Get-WmiObject Win32_Service -Filter "Name='passwordstate-Gateway'").StartName
	"Passwordstate Gateway Service Information" | Out-File $Log3 -append
	"Status: $status1" | Out-File $Log3 -append
	"Log On As User: $pwsuser1" | Out-File $Log3 -append
	"" | Out-File $Log3 -append
    "" | Out-File $Log3 -append

    Write-Host "Finding out information about the Gateway.conf file"
    "Gateway.conf File Contents" | Out-File $Log3 -append
    import-csv "$HostsPath\gateway.conf"  | Out-File $Log3 -append
    "" | Out-File $Log3 -append
    "" | Out-File $Log3 -append
}


#########################################################################################################
# Get permissions on Passwordstate folder
#########################################################################################################
Write-Host "Getting all permissions on the Passwordstate install folder"
"Passwordstate Folder Permissions" | Out-File $Log3 -append
$perms = Get-Acl $installpath | select -expand access
$perms | select IdentityReference, FileSystemRights | Out-File $Log3 -append
"" | Out-File $Log3 -append
"" | Out-File $Log3 -append

#########################################################################################################
# Get TLS Registry Values
#########################################################################################################
Function Get-RegValue {
    [CmdletBinding()]
    Param
    (
        # Registry Path
        [Parameter(Mandatory = $true,
            Position = 0)]
        [string]
        $RegPath,

        # Registry Name
        [Parameter(Mandatory = $true,
            Position = 1)]
        [string]
        $RegName
    )
    $regItem = Get-ItemProperty -Path $RegPath -Name $RegName -ErrorAction Ignore
    $output = "" | select Path, Name, Value
    $output.Path = $RegPath
    $output.Name = $RegName

    If ($regItem -eq $null) {
        $output.Value = "Not Found"
    }
    Else {
        $output.Value = $regItem.$RegName
    }
    $output
}

$regSettings = @()
$regKey = 'HKLM:\SYSTEM\CurrentControlSet\Control\SecurityProviders\SCHANNEL\Protocols\TLS 1.3\Server'
$regSettings += Get-RegValue $regKey 'Enabled'
$regSettings += Get-RegValue $regKey 'DisabledByDefault'


$regKey = 'HKLM:\SYSTEM\CurrentControlSet\Control\SecurityProviders\SCHANNEL\Protocols\TLS 1.3\Client'
$regSettings += Get-RegValue $regKey 'Enabled'
$regSettings += Get-RegValue $regKey 'DisabledByDefault'

$regKey = 'HKLM:\SYSTEM\CurrentControlSet\Control\SecurityProviders\SCHANNEL\Protocols\TLS 1.2\Server'
$regSettings += Get-RegValue $regKey 'Enabled'
$regSettings += Get-RegValue $regKey 'DisabledByDefault'

$regKey = 'HKLM:\SYSTEM\CurrentControlSet\Control\SecurityProviders\SCHANNEL\Protocols\TLS 1.2\Client'
$regSettings += Get-RegValue $regKey 'Enabled'
$regSettings += Get-RegValue $regKey 'DisabledByDefault'

$regKey = 'HKLM:\SYSTEM\CurrentControlSet\Control\SecurityProviders\SCHANNEL\Protocols\TLS 1.1\Server'
$regSettings += Get-RegValue $regKey 'Enabled'
$regSettings += Get-RegValue $regKey 'DisabledByDefault'

$regKey = 'HKLM:\SYSTEM\CurrentControlSet\Control\SecurityProviders\SCHANNEL\Protocols\TLS 1.1\Client'
$regSettings += Get-RegValue $regKey 'Enabled'
$regSettings += Get-RegValue $regKey 'DisabledByDefault'

$regKey = 'HKLM:\SYSTEM\CurrentControlSet\Control\SecurityProviders\SCHANNEL\Protocols\TLS 1.0\Server'
$regSettings += Get-RegValue $regKey 'Enabled'
$regSettings += Get-RegValue $regKey 'DisabledByDefault'

$regKey = 'HKLM:\SYSTEM\CurrentControlSet\Control\SecurityProviders\SCHANNEL\Protocols\TLS 1.0\Client'
$regSettings += Get-RegValue $regKey 'Enabled'
$regSettings += Get-RegValue $regKey 'DisabledByDefault'

$regSettings | Out-File $Log3 -append
"" | Out-File $Log3 -append
"" | Out-File $Log3 -append


"#########################################################################################################" | Out-File $Log4 -append
"# Web.config Information" | Out-File $Log4 -append
"#########################################################################################################" | Out-File $Log4 -append
"" | Out-File $Log4 -append

#########################################################################################################
# Copy the web.config file to a temporary folder
#########################################################################################################
Write-Host "Making a copy of web.config file to a temporary location, so we can work with it without affecting currently active users"
$webConfigFile = "$installpath\web.config"
$destination = $PSScriptroot
Copy-Item $webConfigFile $destination -Force

$destinationFile = (Join-Path $destination "web.config") 
if (Test-Path $destinationFile) { 
        # if $env:TEMP had some special characters in the path name, the path is not valid for usage with aspnet_regiis.exe. So we need the real path 
        $destination = Get-ChildItem $destinationFile | Select-Object -ExpandProperty DirectoryName 
}
"" | Out-File $Log4 -append

#########################################################################################################
# Performing a check to see if the connection string in the web.config file is encrypted
#########################################################################################################
Write-Host "Performing a check to see if the connection string in the web.config file is encrypted, and storing results in variable for use later in the script"
"Check Connection String" | Out-File $Log4 -append
$webConfigFile = Get-Content "$destination\web.config"
if ($webConfigFile -like '*PasswordstateConnectionString*')
{
$ConnEncrypt = $false
"Status: Not encrypted" | Out-File $Log4 -append
"" | Out-File $Log4 -append

}
else
{
$ConnEncrypt = $true
"Status: Encrypted" | Out-File $Log4 -append
"" | Out-File $Log4 -append
}

#########################################################################################################
# Performing a check to see if the App Settings section in the web.config file is encrypted
#########################################################################################################
Write-Host "Performing a check to see if the App Settings section in the web.config file is encrypted, and storing results in variable for use later in the script"
"Check AppSettings" | Out-File $Log4 -append
if ($webConfigFile -like '*Secret1*')
{
$AppEncrypt = $false
"Status: Not encrypted" | Out-File $Log4 -append
"" | Out-File $Log4 -append
}
else
{
$AppEncrypt = $true
"Status: Encrypted" | Out-File $Log4 -append
"" | Out-File $Log4 -append
}

#########################################################################################################
# If the connection string in the web.config file is encrypted, we will now decrypt it
#########################################################################################################
Write-Host "If the connection string in the web.config file is encrypted, we will now decrypt it."
"If the connection string in the web.config file is encrypted, we will now decrypt it." | Out-File $Log4 -append
if ($ConnEncrypt -eq $true)
{
"The Connection String is encrypted, so we'll now decrypt it." | Out-File $Log4 -append
CD C:\Windows\Microsoft.NET\Framework64\v4.0.30319
.\aspnet_regiis.exe -pdf "connectionStrings" "$destination"
"The Connection String was decrypted successfully." | Out-File $Log4 -append
"" | Out-File $Log4 -append
}
else
{
"We were not required to decrypt the Connection String Section of the web.config file" | Out-File $Log4 -append
"" | Out-File $Log4 -append
}

#########################################################################################################
# If the App Settings Section in the web.config file is encrypted, we will now decrypt it
#########################################################################################################
Write-Host "If the App Settings Section in the web.config file is encrypted, we will now decrypt it"

"If the App Settings Section in the web.config file is encrypted, we will now decrypt it" | Out-File $Log4 -append
if ($AppEncrypt -eq $true)
{
"The App Settings section is encrypted, so we'll now decrypt it." | Out-File $Log4 -append
CD C:\Windows\Microsoft.NET\Framework64\v4.0.30319
.\aspnet_regiis.exe -pdf "appSettings" "$destination"
"The App Settings section was decrypted successfully." | Out-File $Log4 -append
"" | Out-File $Log4 -append
}
else
{
"We were not required to decrypt the App Settings Section of the web.config file" | Out-File $Log4 -append
"" | Out-File $Log4 -append
}

#########################################################################################################
# Check to see if Passwordstate 9 is installed
#########################################################################################################
Write-Host "Performing a query to determine if we are running Passwordstate 8, or Passwordstate 9. Required as PassiveNode is no longer available in Passwordstate 9"
"Performing a query to determine if we are running Passwordstate 8, or Passwordstate 9. Required as PassiveNode is no longer available in Passwordstate 9" | Out-File $Log4 -append
$fileversion = $fileversion.replace('.', '')
if ([int]$fileversion -lt '9000')
{
	Write-Host "Customer has Passwordstate 8 installed. We will query the web.config file for PassiveNode status"
	"Note: Customer has Passwordstate 8 installed. We will query the web.config file for PassiveNode status" | Out-File $Log4 -append
    "" | Out-File $Log4 -append
	$MajorVersion = "8"
}
else
{
	Write-Host "Customer has Passwordstate 9 installed. We will query the database for PassiveNode status"
	"Note: Customer has Passwordstate 9 installed. We will query the database for PassiveNode status" | Out-File $Log4 -append
    "" | Out-File $Log4 -append
	$MajorVersion = "9"
}

If ($MajorVersion -eq '8')
{
	Write-Host "Currently looking in the web.config file to check for PassiveNode status from the AppSetting Section"
	"Information about the PassiveNode status out of the AppSettings section of the web.config file" | Out-File $Log4 -append
	$webConfigFile = [xml](Get-Content "$destination\web.config")
	$root = $webConfigFile.get_DocumentElement();
	$root = $root.appsettings.add
	if ($root -eq '')
	{
		"We've tried to query the Web.config file AppSettings but it is encrypted, so We'll now skip this step and pass onto the next system check." | Out-File $Log4 -append
		"" | Out-File $Log4 -append
	}
	else
	{
		$root | Where-Object { $_.Key -eq "PassiveNode" } | Out-File $Log4 -append
		"" | Out-File $Log4 -append
	}
}

#########################################################################################################
# Check Set Up Status in AppSetting section
#########################################################################################################
Write-Host "Currently looking in the web.config file to check for Setup status from the AppSettings Section"
"Information about the Setup Status out of the AppSettings section of the web.config file" | Out-File $Log4 -append
$webConfigFile = [xml](Get-Content "$destination\web.config")
$root = $webConfigFile.get_DocumentElement();
$root = $root.appsettings.add
if ($root -eq '')
{
	"We've tried to query the Web.config file AppSettings but it is encrypted, so We'll now skip this step and pass onto the next system check." | Out-File $Log4 -append
	"" | Out-File $Log4 -append
}
else
{
	$root | Where-Object { $_.Key -eq "SetupStage" } | Out-File $Log4 -append
}

#########################################################################################################
# Querying web.config file to obtain database connection string
#########################################################################################################
Write-Host "Currently looking in the web.config file to see if we can find connection details to database server"
"Information out of the Connection String section of the web.config file" | Out-File $Log4 -append
$webConfigFile = [xml](Get-Content "$destination\web.config")
[string]$string = $webConfigFile.configuration.connectionstrings.add | Select connectionstring

#Clear variable if it has any data
if ($msaaccount -ne $null)
     {
      Clear-Variable -Name msaaccount
     }

if ($string -like "*Integrated Security=SSPI*" -or $string -eq '')
{
	"We've tried to query the Connection String in the Web.config file but it is using an MSA account, so we cannot make connections to the database.  We'll now skip this step and pass onto the next system check." | Out-File $Log4 -append
	"" | Out-File $Log4 -append
	"" | Out-File $Log4 -append
    #Set variable to indicate MSA account is being used
    $msaaccount = "MSAUsed"
}
else
{
    #Split the Connection string into manageable sections
	$string = $string.substring(2)
	$string = $string.substring(0, $string.length - 1)
	$res = $string.Split(";")
	
    #Loop through all sections and obtain connection data
    foreach ($section in $res)
        {
            if ($section -like '*connectionString*'){$connString = $section.replace('connectionString=Data Source=', '')}
            if ($section -like '*Initial Catalog*'){$initialCatalog = $section.replace('Initial Catalog=', '')}
            if ($section -like '*User ID=*'){$userId = $section.replace('User ID=', '')}
            if ($section -like '*Password=*'){$password = $section.replace('Password=', '')}
        }
}

#########################################################################################################
# Write servername, SQL Instance and database to log file
#########################################################################################################	
Write-Host "We have found connection details in the Web.config file"
"SQL Server and SQL Instance: $connString" | Out-File $Log4 -append
"Database Name: $initialCatalog" | Out-File $Log4 -append
"UserName: $userId" | Out-File $Log4 -append
"" | Out-File $Log4 -append
"" | Out-File $Log4 -append


"#########################################################################################################" | Out-File $Log4 -append
"# Database Information" | Out-File $Log4 -append
"#########################################################################################################" | Out-File $Log4 -append
"" | Out-File $Log4 -append

#########################################################################################################
# Get database server time
#########################################################################################################
Write-Host "Getting the current time of the database server"
"Database Server Time:" | Out-File $Log4 -append
if ($connstring -like "*\*")
{
Write-Host "Instance detected"
$DatabaseServer = $connstring.split("\")
$DatabaseServer = $DatabaseServer[0]
}
else
{
Write-Host "No Instance detected"
$DatabaseServer = $connstring
}
invoke-command -ComputerName $DatabaseServer -ScriptBlock {get-date} | Out-File $Log4 -append
"" | Out-File $Log4 -append


#########################################################################################################
# Add all SQL Queries into variables to be called later
#########################################################################################################

#Total count of Shared Password Lists
$SQLQuery1 = "SELECT COUNT(*) AS 'Total Shared Lists' FROM [PasswordLists] WHERE (PasswordLists.PrivatePasswordList = 0) AND (PasswordLists.Folder <> 1)"

#Total count of Folders
$SQLQuery2 = "SELECT COUNT(*) AS Folders FROM [PasswordLists] WHERE (Folder = 1)"

# Total count of Auditing Records	
$SQLQuery3 = "SELECT COUNT(*) AS 'Total Auditing Records' FROM Auditing"
	
#Total count of Users in the system
$SQLQuery4 = "SELECT COUNT(*) AS 'Total Users' FROM UserAccounts"
	
#Total count of Security Groups in the system
$SQLQuery5 = "SELECT COUNT(*) AS 'Total Security Groups' FROM SecurityGroups"
	
#Information from System Settings table
$SQLQuery6 = "SELECT BuildNo, SiteURL, FIPSMode, ADSyncSchedule, RecordingFolder, SelfDestructURL, GatewayURL, GatewayPort, BrowserExtURL FROM SystemSettings"

#Information from the Debug Info table	
$SQLQuery7 = "SELECT DebugDateTime, Category, DebugInformation, EventType FROM DebugInfo"
	
#Information from the Backups table - for Passwordstate 8 instances
$SQLQuery8 = "SELECT BackupFilePath, BackupUserName, ExcludeDBBackup FROM BackupSettings"

#Get the SQL Version
$SQLQuery9 = "SELECT @@Version"

#Get the SQL Server Instance name
$SQLQuery10 = "SELECT @@ServiceName"

#Information from the Licensing table
$SQLQuery11 = "SELECT LicenseType, RegistrationName, NoLicenses, SupportExpires, TrialPeriod, DisplayLabel, TrialDays FROM Licenses"

#Total count of Private Password Lists
$SQLQuery12 = "SELECT COUNT(*) AS 'Total Private Lists' FROM [PasswordLists] WHERE (PasswordLists.PrivatePasswordList = 1) AND (PasswordLists.Folder <> 1)"

#Total count of Empty Shared Password Lists
$SQLQuery13 = "SELECT COUNT(*) AS 'Empty Shared Lists' FROM [PasswordLists] WHERE (PasswordLists.PrivatePasswordList = 0) AND (PasswordLists.Folder <> 1) AND ((SELECT COUNT(PasswordID) FROM [Passwords] PSSWD WHERE (PSSWD.PasswordListID = PasswordLists.PasswordListID) AND (PSSWD.Deleted = 0)) = 0)"

#Total count of Empty Private Password Lists
$SQLQuery14 = "SELECT COUNT(*) AS 'Empty Private Lists' FROM [PasswordLists] WHERE (PasswordLists.PrivatePasswordList = 1) AND (PasswordLists.Folder <> 1) AND ((SELECT COUNT(PasswordID) FROM [Passwords] PSSWD WHERE (PSSWD.PasswordListID = PasswordLists.PasswordListID) AND (PSSWD.Deleted = 0)) = 0)"

#Information from the Browser Extensions table
$SQLQuery15 = "SELECT URL, UserID FROM BrowserExtensionSettings"

#Information about domains
$SQLQuery16 = "SELECT * FROM ADDomains"
	
#Information about Authorized Web Servers
$SQLQuery17 = "SELECT * FROM WebServers"	
	
#Information from the Backups table - for Passwordstate 9 instances
$SQLQuery18 = "SELECT BackupFilePath, BackupDBPath, ZipProtection, ExcludeDBBackup, BackupsToKeep, BackupInterval, BackupStartTime, SQLServerRemote, SQLDNSLookup FROM BackupSettings"

#Total Count of Passwords
$SQLQuery19 = "SELECT COUNT(*) AS Passwords FROM [Passwords]"

#Detailed information about database tables
$SQLQuery20 = @"
SELECT
t.Name AS TableName,
p.rows AS RowCounts,
CAST(ROUND((SUM(a.used_pages) / 128.00), 2) AS NUMERIC(36, 2)) AS Used_MB,
CAST(ROUND((SUM(a.total_pages) - SUM(a.used_pages)) / 128.00, 2) AS NUMERIC(36, 2)) AS Unused_MB,
CAST(ROUND((SUM(a.total_pages) / 128.00), 2) AS NUMERIC(36, 2)) AS Total_MB
FROM sys.tables t
INNER JOIN sys.indexes i ON t.OBJECT_ID = i.object_id
INNER JOIN sys.partitions p ON i.object_id = p.OBJECT_ID AND i.index_id = p.index_id
INNER JOIN sys.allocation_units a ON p.partition_id = a.container_id
INNER JOIN sys.schemas s ON t.schema_id = s.schema_id
GROUP BY t.Name, s.Name, p.Rows
ORDER BY s.Name, t.Name
"@

#Total count of Auditing Archive Records	
$SQLQuery21 = "SELECT COUNT(*) AS 'Total Auditing Archive Records' FROM AuditingArchive"

#Information about Email Notification Groups
$SQLQuery22 = @"
SELECT NotificationGroup as 'Notification Group Name'
,UserID as 'Username'
,SecurityGroupName as 'Security Group Name'
FROM EmailNotificationGroups
Join EmailNotificationGroupsACL ON EmailNotificationGroupsACL.EmailNotificationGroupID = EmailNotificationGroups.EmailNotificationGroupID
LEFT OUTER JOIN SecurityGroups ON SecurityGroups.SecurityGroupID = EmailNotificationGroupsACL.SecurityGroupID
ORDER BY NotificationGroup
"@

#Information about Email Templates
$SQLQuery23 = "SELECT Category, Enabled, GenericMailboxes FROM EmailTemplates"

#Information about Browser Extension settings under Administration tab
$SQLQuery25 = "SELECT BEAutoLogoutAfterClose as 'Logout After Browser Closes', BEAutoLogoutIdle as 'Idle Timeout Value', BrowserExtURL as 'URL', BEMenuURLs as 'Use Base URL from System Settings' FROM SystemSettings"

#Return very last entry from Auditing table
$SQLQuery26 = "SELECT TOP 1 AuditDate as 'Oldest Auditing Record' FROM Auditing ORDER BY AuditDate ASC"

#Information about User Account Policies
$SQLQuery27 = @"
SELECT PolicyName as 'Policy Name'
      ,PolicyDescription 'Policy Description'
      ,Enabled
      ,AuthenticationOption
      ,UserID as 'Username'
	  ,SecurityGroupName as 'Security Group Name'

FROM UserAccountPolicies
JOIN UserAccountPoliciesACL ON UserAccountPolicies.UserAccountPolicyID = UserAccountPoliciesACL.UserAccountPolicyID
LEFT OUTER JOIN SecurityGroups ON SecurityGroups.SecurityGroupID = UserAccountPoliciesACL.SecurityGroupID
WHERE AuthenticationOption <> ''
ORDER BY PolicyName
"@

#Information about database size
$SQLQuery28 = @"
EXEC sp_spaceused
"@

#Information about authentication option set under System Settings
$SQLQuery29 = "SELECT AuthenticationOption FROM SystemSettings"

#Last successful backup date
$SQLQuery30 = "SELECT TOP 1 BackupDate as 'Last Backup Date' FROM BackupStatus ORDER BY BackupStatusID DESC"

#Blocked Email Notifications
$SQLQuery31 = @"
SELECT NotificationGroup as 'Notification Group Name', Category as 'Blocked Email Notification' 
FROM BlockedEmailNotificationGroups
Join EmailNotificationGroups ON EmailNotificationGroups.EmailNotificationGroupID = BlockedEmailNotificationGroups.EmailNotificationGroupID
"@

#Queued Emails
$SQLQuery32 = "SELECT COUNT(*) as 'Queued Email Count' FROM QueuedEmail"

#Information about domains
$SQLQuery33 = "SELECT FQDN FROM ADDomains"

#Allowed IP Ranges check
$SQLQuery34 = "SELECT AllowedIPRanges FROM SystemSettings"

#Base URL check
$SQLQuery35 = "SELECT SiteURL FROM SystemSettings"

#Allowed IP Ranges check
$SQLQuery36 = "SELECT DeniedIPAuthOption FROM SystemSettings"


Write-Host "Retrieving some information from the SQL Database"
    
    $Server = $connstring
	$database = $initialcatalog
	$Connection = New-Object System.Data.SQLClient.SQLConnection
	$Connection.ConnectionString = "server=$($Server);database=$($Database);User ID=$userID;Password=$password"
	$Connection.Open()
	$Command = New-Object System.Data.SQLClient.SQLCommand
	$Command.Connection = $Connection

    "SQL Server Version" | Out-File $Log4 -append
	$Command.CommandText = $SQLQuery9
	$Reader = $Command.ExecuteReader()
	$Datatable = New-Object System.Data.DataTable
	$Datatable.Load($Reader)
	$datatable | Format-List | Out-File $Log4 -append
	"" | Out-File $Log4 -append
    
    "SQL Server Instance Name" | Out-File $Log4 -append
	$Command.CommandText = $SQLQuery10
	$Reader = $Command.ExecuteReader()
	$Datatable = New-Object System.Data.DataTable
	$Datatable.Load($Reader)
	$datatable | Format-List | Out-File $Log4 -append
	"" | Out-File $Log4 -append

	"Information about Authorized Web Servers" | Out-File $Log4 -append
	$Command.CommandText = $SQLQuery17
	$Reader = $Command.ExecuteReader()
	$Datatable = New-Object System.Data.DataTable
	$Datatable.Load($Reader)
	$datatable | ft -autosize | Out-File $Log4 -append

	"Information about Active Directory Domains" | Out-File $Log4 -append
    "" | Out-File $Log4 -append
	$Command.CommandText = $SQLQuery16
	$Reader = $Command.ExecuteReader()
	$Datatable = New-Object System.Data.DataTable
	$Datatable.Load($Reader)
	foreach ($Domain in $Datatable)
        {
        $Netbios = $Domain.ADDomainNetBios
        $FQDN = $Domain.FQDN
        $DefaultDomain = $Domain.DefaultDomain
        $DomainController = $Domain.DomainControllerFQDN
        $LDAPQueryString = $Domain.ADDomainLDAP
        $Authentication = $Domain.UsedForAuthentication
        $Protocol = $Domain.Protocol

        if ($Protocol -eq 0){$ConnectProtocol = "LDAP"}
        if ($Protocol -eq 1){$ConnectProtocol = "LDAPS"}
        if ($Protocol -eq 2){$ConnectProtocol = "Kerberos"}

        "Netbios                  : $Netbios" | Out-File $Log4 -append
        "FQDN                     : $FQDN" | Out-File $Log4 -append
        "LDAP Query String        : $LDAPQueryString" | Out-File $Log4 -append
        "Domain Controller        : $DomainController" | Out-File $Log4 -append
        "Default Domain           : $DefaultDomain" | Out-File $Log4 -append
        "Used for Authentication  : $Authentication" | Out-File $Log4 -append
        "Connection Protocol      : $ConnectProtocol" | Out-File $Log4 -append
        "" | Out-File $Log4 -append
        }
    "" | Out-File $Log4 -append

    "Information From System Settings" | Out-File $Log4 -append
    "" | Out-File $Log4 -append
	$Command.CommandText = $SQLQuery6
	$Reader = $Command.ExecuteReader()
	$Datatable = New-Object System.Data.DataTable
	$Datatable.Load($Reader)
	$BuildNumber = $datatable.buildno
    $BaseUrl = $datatable.SiteURL
    
    $SDURL = $datatable.SelfDestructURL
    $GatewayURL = $datatable.GatewayURL
    $RecFolder = $datatable.RecordingFolder
    $GatewayPort = $datatable.GatewayPort
    $Encryption = $datatable.FIPSMode
    if ($Encryption -eq $false){$EncryptModel = "FIPS Encryption"}else{$EncryptModel = "AES 256 Bit Encryption"}
    $ADSyncSched = $datatable.ADSyncASchedule
    $BEExtURL = $datatable.BrowserExtURL

    "Database Build Number                     : $BuildNumber" | Out-File $Log4 -append
    "Base URL                                  : $BaseUrl" | Out-File $Log4 -append
    "Self Destruct URL                         : $SDURL" | Out-File $Log4 -append
    "Browser Based Gateway URL                 : $GatewayURL" | Out-File $Log4 -append
    "Browser Extension URL                     : $BEExtURL" | Out-File $Log4 -append
    "Browser Based Gateway Recording Folder    : $RecFolder" | Out-File $Log4 -append
    "Browser Based Gateway Port                : $GatewayPort" | Out-File $Log4 -append
    "Encryption Model                          : $EncryptModel" | Out-File $Log4 -append
    "Active Directory Sync Schedule            : $ADSyncSched" | Out-File $Log4 -append
	"" | Out-File $Log4 -append

     "Allowed IP Ranges From System Settings" | Out-File $Log4 -append
	$Command.CommandText = $SQLQuery34
	$Reader = $Command.ExecuteReader()
	$Datatable = New-Object System.Data.DataTable
	$Datatable.Load($Reader)
	if ([string]$Datatable.AllowedIPRanges -eq '')
    { "There are no Allowed IP Ranges configured" | Out-File $Log4 -append}
    else
    { "The system is configured with one or more allowed IP Ranges, but values are encrypted in the database so Server Info cannot retrieve them" | Out-File $Log4 -append}
	"" | Out-File $Log4 -append
    "" | Out-File $Log4 -append
	
    If ($MajorVersion -eq '8')
	{
    "Information From Backups Table" | Out-File $Log4 -append
	$Command.CommandText = $SQLQuery8
	$Reader = $Command.ExecuteReader()
	$Datatable = New-Object System.Data.DataTable
	$Datatable.Load($Reader)
	$datatable | ft -autosize | Out-File $Log4 -append
	"" | Out-File $Log4 -append
    }

    If ($MajorVersion -eq '9')
	{
    "Information From Backups Table" | Out-File $Log4 -append
	$Command.CommandText = $SQLQuery18
	$Reader = $Command.ExecuteReader()
	$Datatable = New-Object System.Data.DataTable
	$Datatable.Load($Reader)
	$datatable | ft -autosize | Out-File $Log4 -append
	"" | Out-File $Log4 -append

	$Command.CommandText = $SQLQuery30
	$Reader = $Command.ExecuteReader()
	$Datatable = New-Object System.Data.DataTable
	$Datatable.Load($Reader)
	$datatable | ft -autosize | Out-File $Log4 -append
	"" | Out-File $Log4 -append
    }

    "Information Browser Extension settings in Administration area" | Out-File $Log4 -append
	$Command.CommandText = $SQLQuery25
	$Reader = $Command.ExecuteReader()
	$Datatable = New-Object System.Data.DataTable
	$Datatable.Load($Reader)
	$datatable | ft -autosize | Out-File $Log4 -append

	"Total count of Shared PasswordLists" | Out-File $Log4 -append
	$Command.CommandText = $SQLQuery1
	$Reader = $Command.ExecuteReader()
	$Datatable = New-Object System.Data.DataTable
	$Datatable.Load($Reader)
	$datatable | ft -autosize | Out-File $Log4 -append

    "Total count of Private PasswordLists" | Out-File $Log4 -append
	$Command.CommandText = $SQLQuery12
	$Reader = $Command.ExecuteReader()
	$Datatable = New-Object System.Data.DataTable
	$Datatable.Load($Reader)
	$datatable | ft -autosize | Out-File $Log4 -append
	
    "Total count of Empty Shared PasswordLists" | Out-File $Log4 -append
	$Command.CommandText = $SQLQuery13
	$Reader = $Command.ExecuteReader()
	$Datatable = New-Object System.Data.DataTable
	$Datatable.Load($Reader)
	$datatable | ft -autosize | Out-File $Log4 -append

    "Total count of Empty Private PasswordLists" | Out-File $Log4 -append
	$Command.CommandText = $SQLQuery14
	$Reader = $Command.ExecuteReader()
	$Datatable = New-Object System.Data.DataTable
	$Datatable.Load($Reader)
	$datatable | ft -autosize | Out-File $Log4 -append

	"Total Count of  Folders" | Out-File $Log4 -append
	$Command.CommandText = $SQLQuery2
	$Reader = $Command.ExecuteReader()
	$Datatable = New-Object System.Data.DataTable
	$Datatable.Load($Reader)
	$datatable | ft -autosize | Out-File $Log4 -append

    "Total Count of  Passwords" | Out-File $Log4 -append
	$Command.CommandText = $SQLQuery19
	$Reader = $Command.ExecuteReader()
	$Datatable = New-Object System.Data.DataTable
	$Datatable.Load($Reader)
	$datatable | ft -autosize | Out-File $Log4 -append
	
    "Total count of Queued Emails" | Out-File $Log4 -append
	$Command.CommandText = $SQLQuery32
	$Reader = $Command.ExecuteReader()
	$Datatable = New-Object System.Data.DataTable
	$Datatable.Load($Reader)
	$datatable | ft -autosize | Out-File $Log4 -append

    "Information about Ignored URLs in the Browser Extensions Table" | Out-File $Log9 -append
	$Command.CommandText = $SQLQuery15
	$Reader = $Command.ExecuteReader()
	$Datatable = New-Object System.Data.DataTable
	$Datatable.Load($Reader)
	$datatable | ft -autosize | Out-File $Log9 -append


    "Oldest record in Auditing Table" | Out-File $Log11 -append
	$Command.CommandText = $SQLQuery26
	$Reader = $Command.ExecuteReader()
	$Datatable = New-Object System.Data.DataTable
	$Datatable.Load($Reader)
	$datatable | ft -autosize | Out-File $Log11 -append

	"Count of Auditing records in main Auditing Table" | Out-File $Log11 -append
	$Command.CommandText = $SQLQuery3
	$Reader = $Command.ExecuteReader()
	$Datatable = New-Object System.Data.DataTable
	$Datatable.Load($Reader)
	$datatable | ft -autosize | Out-File $Log11 -append

    "Count of Auditing records in Auditing Archive Table" | Out-File $Log11 -append
	$Command.CommandText = $SQLQuery21
	$Reader = $Command.ExecuteReader()
	$Datatable = New-Object System.Data.DataTable
	$Datatable.Load($Reader)
	$datatable | ft -autosize | Out-File $Log11 -append
	
	"Count of Total users in the system" | Out-File $Log4 -append
	$Command.CommandText = $SQLQuery4
	$Reader = $Command.ExecuteReader()
	$Datatable = New-Object System.Data.DataTable
	$Datatable.Load($Reader)
	$datatable | ft -autosize | Out-File $Log4 -append
	
	"Count of Total Security Groups" | Out-File $Log4 -append
	$Command.CommandText = $SQLQuery5
	$Reader = $Command.ExecuteReader()
	$Datatable = New-Object System.Data.DataTable
	$Datatable.Load($Reader)
	$datatable | ft -autosize | Out-File $Log4 -append

    "Information From Licensing table" | Out-File $Log4 -append
	$Command.CommandText = $SQLQuery11
	$Reader = $Command.ExecuteReader()
	$Datatable = New-Object System.Data.DataTable
	$Datatable.Load($Reader)
	$datatable | Format-List | Out-File $Log4 -append
	"" | Out-File $Log4 -append
    
    "Total Database Size" | Out-File $Log11 -append
	$Command.CommandText = $SQLQuery28
	$Reader = $Command.ExecuteReader()
	$Datatable = New-Object System.Data.DataTable
	$Datatable.Load($Reader)
	$datatable | Format-List | Out-File $Log11 -append
	"" | Out-File $Log11 -append
    
    #we need to close and reopen the SQL connection now, otherwise running two databse sdize queries in a row will not work
    $Connection.Close()

    $Server = $connstring
	$database = $initialcatalog
	$Connection = New-Object System.Data.SQLClient.SQLConnection
	$Connection.ConnectionString = "server=$($Server);database=$($Database);User ID=$userID;Password=$password"
	$Connection.Open()
	$Command = New-Object System.Data.SQLClient.SQLCommand
	$Command.Connection = $Connection

    "Database Table Sizes" | Out-File $Log11 -append
	$Command.CommandText = $SQLQuery20
	$Reader = $Command.ExecuteReader()
	$Datatable = New-Object System.Data.DataTable
	$Datatable.Load($Reader)
	$datatable | Format-List | Out-File $Log11 -append
	"" | Out-File $Log11 -append
    
     "Details about Email Templates" | Out-File $Log7 -append
	$Command.CommandText = $SQLQuery23
	$Reader = $Command.ExecuteReader()
	$Datatable = New-Object System.Data.DataTable
	$Datatable.Load($Reader)
	$datatable | Out-File $Log7 -append
	"" | Out-File $Log7 -append

    "Details for Email Notification Groups" | Out-File $Log7 -append
	$Command.CommandText = $SQLQuery22
	$Reader = $Command.ExecuteReader()
	$Datatable = New-Object System.Data.DataTable
	$Datatable.Load($Reader)
	$datatable | Out-File $Log7 -append
	"" | Out-File $Log7 -append

    "Details about Blocked Email Notifications" | Out-File $Log7 -append
	$Command.CommandText = $SQLQuery31
	$Reader = $Command.ExecuteReader()
	$Datatable = New-Object System.Data.DataTable
	$Datatable.Load($Reader)
	$datatable | Out-File $Log7 -append
	"" | Out-File $Log7 -append

    "Details About System Wide Authentication Option" | Out-File $Log8 -append
    "" | Out-File $Log8 -append
	$Command.CommandText = $SQLQuery29
	$Reader = $Command.ExecuteReader()
	$Datatable = New-Object System.Data.DataTable
	$Datatable.Load($Reader)
	$Auth = $Datatable.AuthenticationOption
    $AuthResult = switch ($Auth) {

    "1" {"Manual Login Authentication"}
    "2" {"Manual Login and Google Authenticator"}
    "3" {"Manual Login and RSA SecurID Authentication"}
    "4" {"Manual Login and ScramblePad Authentication"}
    "9" {"Manual Login and Email Temporary Pin Code"}
    "13" {"Manual Login and Duo Authentication"}
    "17" {"Manual Login and One-Time Authentication"}
    "20" {"Manual Login and RADIUS Authentication"}
    "22" {"Manual Login and YubiKey Authentication"}
    "0" {"AD Single Sign-On Authentication"}
    "5" {"AD Single Sign-On and Google Authenticator"}
    "6" {"AD Single Sign-On and RSA SecurID Authentication"}
    "7" {"AD Single Sign-On and ScramblePad Authentication"}
    "10" {"AD Single Sign-On and Email Temporary Pin Code"}
    "14" {"AD Single Sign-On and Duo Authentication"}
    "16" {"AD Single Sign-On and One-Time Authentication"}
    "21" {"AD Single Sign-On and RADIUS Authentication"}
    "23" {"AD Single Sign-On and YubiKey Authentication"}
    "19" {"SAML Authentication"}
    }
    "System Wide Authentication Option: $AuthResult" | Out-File $Log8 -append
	"" | Out-File $Log8 -append
    "" | Out-File $Log8 -append

    "Details About User Account Policies" | Out-File $Log8 -append
	$Command.CommandText = $SQLQuery27
	$Reader = $Command.ExecuteReader()
    $Datatable = New-Object System.Data.DataTable
    $dataTable.Columns.Add((New-Object System.Data.DataColumn "Policy Name", ([string])))
    $dataTable.Columns.Add((New-Object System.Data.DataColumn "Policy Description", ([string])))
    $dataTable.Columns.Add((New-Object System.Data.DataColumn "Enabled", ([string])))
    $dataTable.Columns.Add((New-Object System.Data.DataColumn "Authentication Option", ([string])))
    $dataTable.Columns.Add((New-Object System.Data.DataColumn "Username", ([string])))
    $dataTable.Columns.Add((New-Object System.Data.DataColumn "Security Group Name", ([string])))
    $Datatable.Load($Reader)

    foreach($dataRow in $Datatable)
    {
        
        $Auth = $dataRow.AuthenticationOption
        $AuthResult = switch ($Auth) {

        "1" {"Manual Login Authentication"}
        "2" {"Manual Login and Google Authenticator"}
        "3" {"Manual Login and RSA SecurID Authentication"}
        "4" {"Manual Login and ScramblePad Authentication"}
        "9" {"Manual Login and Email Temporary Pin Code"}
        "13" {"Manual Login and Duo Authentication"}
        "17" {"Manual Login and One-Time Authentication"}
        "20" {"Manual Login and RADIUS Authentication"}
        "22" {"Manual Login and YubiKey Authentication"}
        "0" {"AD Single Sign-On Authentication"}
        "5" {"AD Single Sign-On and Google Authenticator"}
        "6" {"AD Single Sign-On and RSA SecurID Authentication"}
        "7" {"AD Single Sign-On and ScramblePad Authentication"}
        "10" {"AD Single Sign-On and Email Temporary Pin Code"}
        "14" {"AD Single Sign-On and Duo Authentication"}
        "16" {"AD Single Sign-On and One-Time Authentication"}
        "21" {"AD Single Sign-On and RADIUS Authentication"}
        "23" {"AD Single Sign-On and YubiKey Authentication"}
        "19" {"SAML Authentication"}
        }
        $dataRow[3] = $Authresult
        $datatable.AcceptChanges()
    }

    $dataTable.Columns.Remove('authenticationoption')
    $datatable | ft -autosize | Out-File $Log8 -append
    "" | Out-File $Log8 -append

    #This next database query is specifically used to do an NSLookup and Trace Route
    "Information about Base URL from System Settings" | Out-File $Log2 -append
	$Command.CommandText = $SQLQuery35
	$Reader = $Command.ExecuteReader()
	$Datatable = New-Object System.Data.DataTable
	$Datatable.Load($Reader)
    $FullBaseURL = $datatable.SiteURL
	$BaseURL = $datatable.SiteURL
    ($BaseURL = $BaseURL.split("/")[-1]) > $null 2>&1
    $BaseURLResults = Resolve-DnsName -Name $BaseURL
    $BaseURLServer = $BaseURLResults.NameHost
    $BaseURLIP = $BaseURLResults.IPAddress
    "BaseURL: $FullBaseURL" | Out-File $Log2 -append
    "" | Out-File $Log2 -append
    "Attempting to perform a NSLookup of $BaseURL" | Out-File $Log2 -append
    "Server Name: $BaseURLServer" | Out-File $Log2 -append
    "IP Address: $BaseURLIP" | Out-File $Log2 -append
	"" | Out-File $Log2 -append
    "" | Out-File $Log2 -append

    #Starting Trace Route for URL
    $BaseURLTraceRoute = Tracert $BaseURL
    "Attempting to perform a Trace Route to " + $FullBaseURL + ":" | Out-File $Log2 -append
    $BaseURLTraceRoute | Out-File $Log2 -append
    "" | Out-File $Log2 -append
    "" | Out-File $Log2 -append
    "" | Out-File $Log2 -append

"#########################################################################################################" | Out-File $Log5 -append
"# Debug Info Information" | Out-File $Log5 -append
"#########################################################################################################" | Out-File $Log5 -append
    if ($msaaccount -eq "MSAUsed")
    {
    "We've previously detected that an MSA account is being used on the website, so we cannot query the Error console." | Out-File $Log5 -append
    "" | Out-File $Log5 -append
    "" | Out-File $Log5 -append
    }
    else
    {
	    $Command.CommandText = $SQLQuery7
	    $Reader = $Command.ExecuteReader()
	    $Datatable = New-Object System.Data.DataTable
	    $Datatable.Load($Reader)
	    $datatable | Format-List | Out-File $Log5 -append
	    "" | Out-File $Log5 -append
	
        $Connection.Close()
    }

"#########################################################################################################" | Out-File $Log6 -append
"# Event Log Information" | Out-File $Log6 -append
"#########################################################################################################" | Out-File $Log6 -append
"" | Out-File $Log6 -append

#########################################################################################################
# Get all Application event log errors within the last 7 days
#########################################################################################################
Write-Host "Listing all errors in the Application Event log"
Set-Variable -Name EventAgeDays -Value 7 #we will take events for the latest 7 days
Set-Variable -Name CompArr -Value @("localhost") # replace it with your server names
Set-Variable -Name LogNames -Value @("Application") # Checking app and system logs
Set-Variable -Name EventTypes -Value @("2") # Loading only Errors and Warnings
Set-Variable -Name ExportFolder -Value $Path

$el_c = @() #consolidated error log
$now = get-date
$startdate = $now.adddays(- $EventAgeDays)
$ExportFile = $ExportFolder + "el" + $now.ToString("yyyy-MM-dd---hh-mm-ss") + ".csv" # we cannot use standard delimiters like ":"

foreach ($comp in $CompArr)
{
	foreach ($log in $LogNames)
	{
		$el = get-winevent -ComputerName $comp -FilterHashtable @{ logname = "$log"; level = $eventtypes; starttime = $startdate } -ErrorAction SilentlyContinue
		$el_c += $el #consolidating
	}
}
$el_sorted = $el_c | Sort-Object TimeCreated -descending #sort by time

"All Application Event Log Errors Within 7 Days are Listed Below" | Out-File $Log6 -append
$el_sorted | Select LevelDisplayName, TimeCreated, ProviderName, ID, MachineName, Message | Format-List | Out-File $Log6 -append
"" | Out-File $Log6 -append

#########################################################################################################
# Get all Application event log entries for Passwordstate upgrade
#########################################################################################################
"Event Log Activity for Installs and Upgrades are Listed Below" | Out-File $Log6 -append
$PasswordstateInstaller = Get-EventLog application | select timewritten, source, message | where {$_.Source -eq "PasswordstateInstaller"}
If ($PasswordstateInstaller -eq $null)
{
"No Activity to List" | Out-File $Log6 -append
"" | Out-File $Log6 -append
"" | Out-File $Log6 -append
}
Else
{
$PasswordstateInstaller | Out-File $Log6 -append
"" | Out-File $Log6 -append
"" | Out-File $Log6 -append
}

"#########################################################################################################" | Out-File $Log6 -append
"# Powershell Error Information" | Out-File $Log6 -append
"#########################################################################################################" | Out-File $Log6 -append
"" | Out-File $Log6 -append

#########################################################################################################
# Output any Powershell errors that have occured during session
#########################################################################################################
Write-Host "Output any Powershell errors that have occured during session"
"Output any Powershell errors that have occured during session" | Out-File $Log6 -append
if ($error.Count -ge '1')
{
$error | Out-File $Log6 -append
"" | Out-File $Log6 -append
"" | Out-File $Log6 -append
}
Else
{
"There are no Powershell errors to report" | Out-File $Log6 -append
"" | Out-File $Log6 -append
"" | Out-File $Log6 -append
}

#########################################################################################################
# Redact sensitive contents of web.config file
#########################################################################################################
Write-Host "Redacting contents of web.config file"
"Begin redacting web.config file to remove sensitive information from Connection String and AppSettings section." | Out-File $Log6 -append
$TempWeb = "$Path\web.config"

$SourceFile = Get-Content $TempWeb -ErrorAction Stop 
$SourceFile -replace "(?<=Password=).+?(?=;)", "[redacted]" | Set-Content "$TempWeb"

$SourceFile = Get-Content $TempWeb -ErrorAction Stop 
$SourceFile -replace "(?<=Password=).+?(?=`")", "[redacted]" | Set-Content "$TempWeb"

$SourceFile = Get-Content $TempWeb -ErrorAction Stop 
$SourceFile -replace "(?<=Secret1`" value=`").+?(?=`")", "[redacted]" | Set-Content "$TempWeb"

$SourceFile = Get-Content $TempWeb -ErrorAction Stop 
$SourceFile -replace "(?<=Secret2`" value=`").+?(?=`")", "[redacted]" | Set-Content "$TempWeb"

"Redacting of web.config file complete." | Out-File $Log6 -append
"" | Out-File $Log6 -append
"" | Out-File $Log6 -append

#########################################################################################################
# Get the end time of the script
#########################################################################################################
Write-Host "Getting the end time of the script"
$End = get-date
"Script finish time is $End" | Out-File $Log6 -append
"" | Out-File $Log6 -append
"" | Out-File $Log6 -append

#########################################################################################################
# Compress text file to Reduce size
#########################################################################################################
Write-Host "Testing whether the compress-archive commandlet is available in Powershell"
if ($host.version.Major -ge 5)
{
Write-Host "Compressing the text file"
"Currently Compressing the output file - The script has now ended" | Out-File $Log6 -append
$compress = @{
LiteralPath= $Log1, $Log2, $Log3, $Log4, $Log5, $Log6, $Log7, $Log8 ,$Log9 ,$Log10 ,$Log11, "$Path\web.config"
CompressionLevel = "Optimal"
DestinationPath = "$Path\ServerInfo.zip"
}
Compress-Archive @compress -Force

#########################################################################################################
# Clean up Files from Disk
#########################################################################################################
Remove-Item $Log1 -Force
Remove-Item $Log2 -Force
Remove-Item $Log3 -Force
Remove-Item $Log4 -Force
Remove-Item $Log5 -Force
Remove-Item $Log6 -Force
Remove-Item $Log7 -Force
Remove-Item $Log8 -Force
Remove-Item $Log9 -Force
Remove-Item $Log10 -Force
Remove-Item $Log11 -Force
Remove-Item $Path\web.config -Force

#Add pop up to confirm script has complete
$wshell = New-Object -ComObject Wscript.Shell
$wshell.Popup("Script is complete. Please email $Path\ServerInfo.zip to support@clickstudios.com.au",0,"Process Complete",0x0) | Out-Null
}

Break



# SIG # Begin signature block
# MIIoGQYJKoZIhvcNAQcCoIIoCjCCKAYCAQExCzAJBgUrDgMCGgUAMGkGCisGAQQB
# gjcCAQSgWzBZMDQGCisGAQQBgjcCAR4wJgIDAQAABBAfzDtgWUsITrck0sYpfvNR
# AgEAAgEAAgEAAgEAAgEAMCEwCQYFKw4DAhoFAAQUK8uBMAUpz3vC07V/8bzUlo/B
# fACggiFBMIIFjTCCBHWgAwIBAgIQDpsYjvnQLefv21DiCEAYWjANBgkqhkiG9w0B
# AQwFADBlMQswCQYDVQQGEwJVUzEVMBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYD
# VQQLExB3d3cuZGlnaWNlcnQuY29tMSQwIgYDVQQDExtEaWdpQ2VydCBBc3N1cmVk
# IElEIFJvb3QgQ0EwHhcNMjIwODAxMDAwMDAwWhcNMzExMTA5MjM1OTU5WjBiMQsw
# CQYDVQQGEwJVUzEVMBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYDVQQLExB3d3cu
# ZGlnaWNlcnQuY29tMSEwHwYDVQQDExhEaWdpQ2VydCBUcnVzdGVkIFJvb3QgRzQw
# ggIiMA0GCSqGSIb3DQEBAQUAA4ICDwAwggIKAoICAQC/5pBzaN675F1KPDAiMGkz
# 7MKnJS7JIT3yithZwuEppz1Yq3aaza57G4QNxDAf8xukOBbrVsaXbR2rsnnyyhHS
# 5F/WBTxSD1Ifxp4VpX6+n6lXFllVcq9ok3DCsrp1mWpzMpTREEQQLt+C8weE5nQ7
# bXHiLQwb7iDVySAdYyktzuxeTsiT+CFhmzTrBcZe7FsavOvJz82sNEBfsXpm7nfI
# SKhmV1efVFiODCu3T6cw2Vbuyntd463JT17lNecxy9qTXtyOj4DatpGYQJB5w3jH
# trHEtWoYOAMQjdjUN6QuBX2I9YI+EJFwq1WCQTLX2wRzKm6RAXwhTNS8rhsDdV14
# Ztk6MUSaM0C/CNdaSaTC5qmgZ92kJ7yhTzm1EVgX9yRcRo9k98FpiHaYdj1ZXUJ2
# h4mXaXpI8OCiEhtmmnTK3kse5w5jrubU75KSOp493ADkRSWJtppEGSt+wJS00mFt
# 6zPZxd9LBADMfRyVw4/3IbKyEbe7f/LVjHAsQWCqsWMYRJUadmJ+9oCw++hkpjPR
# iQfhvbfmQ6QYuKZ3AeEPlAwhHbJUKSWJbOUOUlFHdL4mrLZBdd56rF+NP8m800ER
# ElvlEFDrMcXKchYiCd98THU/Y+whX8QgUWtvsauGi0/C1kVfnSD8oR7FwI+isX4K
# Jpn15GkvmB0t9dmpsh3lGwIDAQABo4IBOjCCATYwDwYDVR0TAQH/BAUwAwEB/zAd
# BgNVHQ4EFgQU7NfjgtJxXWRM3y5nP+e6mK4cD08wHwYDVR0jBBgwFoAUReuir/SS
# y4IxLVGLp6chnfNtyA8wDgYDVR0PAQH/BAQDAgGGMHkGCCsGAQUFBwEBBG0wazAk
# BggrBgEFBQcwAYYYaHR0cDovL29jc3AuZGlnaWNlcnQuY29tMEMGCCsGAQUFBzAC
# hjdodHRwOi8vY2FjZXJ0cy5kaWdpY2VydC5jb20vRGlnaUNlcnRBc3N1cmVkSURS
# b290Q0EuY3J0MEUGA1UdHwQ+MDwwOqA4oDaGNGh0dHA6Ly9jcmwzLmRpZ2ljZXJ0
# LmNvbS9EaWdpQ2VydEFzc3VyZWRJRFJvb3RDQS5jcmwwEQYDVR0gBAowCDAGBgRV
# HSAAMA0GCSqGSIb3DQEBDAUAA4IBAQBwoL9DXFXnOF+go3QbPbYW1/e/Vwe9mqyh
# hyzshV6pGrsi+IcaaVQi7aSId229GhT0E0p6Ly23OO/0/4C5+KH38nLeJLxSA8hO
# 0Cre+i1Wz/n096wwepqLsl7Uz9FDRJtDIeuWcqFItJnLnU+nBgMTdydE1Od/6Fmo
# 8L8vC6bp8jQ87PcDx4eo0kxAGTVGamlUsLihVo7spNU96LHc/RzY9HdaXFSMb++h
# UD38dglohJ9vytsgjTVgHAIDyyCwrFigDkBjxZgiwbJZ9VVrzyerbHbObyMt9H5x
# aiNrIv8SuFQtJ37YOtnwtoeW/VvRXKwYw02fc7cBqZ9Xql4o4rmUMIIGrjCCBJag
# AwIBAgIQBzY3tyRUfNhHrP0oZipeWzANBgkqhkiG9w0BAQsFADBiMQswCQYDVQQG
# EwJVUzEVMBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYDVQQLExB3d3cuZGlnaWNl
# cnQuY29tMSEwHwYDVQQDExhEaWdpQ2VydCBUcnVzdGVkIFJvb3QgRzQwHhcNMjIw
# MzIzMDAwMDAwWhcNMzcwMzIyMjM1OTU5WjBjMQswCQYDVQQGEwJVUzEXMBUGA1UE
# ChMORGlnaUNlcnQsIEluYy4xOzA5BgNVBAMTMkRpZ2lDZXJ0IFRydXN0ZWQgRzQg
# UlNBNDA5NiBTSEEyNTYgVGltZVN0YW1waW5nIENBMIICIjANBgkqhkiG9w0BAQEF
# AAOCAg8AMIICCgKCAgEAxoY1BkmzwT1ySVFVxyUDxPKRN6mXUaHW0oPRnkyibaCw
# zIP5WvYRoUQVQl+kiPNo+n3znIkLf50fng8zH1ATCyZzlm34V6gCff1DtITaEfFz
# sbPuK4CEiiIY3+vaPcQXf6sZKz5C3GeO6lE98NZW1OcoLevTsbV15x8GZY2UKdPZ
# 7Gnf2ZCHRgB720RBidx8ald68Dd5n12sy+iEZLRS8nZH92GDGd1ftFQLIWhuNyG7
# QKxfst5Kfc71ORJn7w6lY2zkpsUdzTYNXNXmG6jBZHRAp8ByxbpOH7G1WE15/teP
# c5OsLDnipUjW8LAxE6lXKZYnLvWHpo9OdhVVJnCYJn+gGkcgQ+NDY4B7dW4nJZCY
# OjgRs/b2nuY7W+yB3iIU2YIqx5K/oN7jPqJz+ucfWmyU8lKVEStYdEAoq3NDzt9K
# oRxrOMUp88qqlnNCaJ+2RrOdOqPVA+C/8KI8ykLcGEh/FDTP0kyr75s9/g64ZCr6
# dSgkQe1CvwWcZklSUPRR8zZJTYsg0ixXNXkrqPNFYLwjjVj33GHek/45wPmyMKVM
# 1+mYSlg+0wOI/rOP015LdhJRk8mMDDtbiiKowSYI+RQQEgN9XyO7ZONj4KbhPvbC
# dLI/Hgl27KtdRnXiYKNYCQEoAA6EVO7O6V3IXjASvUaetdN2udIOa5kM0jO0zbEC
# AwEAAaOCAV0wggFZMBIGA1UdEwEB/wQIMAYBAf8CAQAwHQYDVR0OBBYEFLoW2W1N
# hS9zKXaaL3WMaiCPnshvMB8GA1UdIwQYMBaAFOzX44LScV1kTN8uZz/nupiuHA9P
# MA4GA1UdDwEB/wQEAwIBhjATBgNVHSUEDDAKBggrBgEFBQcDCDB3BggrBgEFBQcB
# AQRrMGkwJAYIKwYBBQUHMAGGGGh0dHA6Ly9vY3NwLmRpZ2ljZXJ0LmNvbTBBBggr
# BgEFBQcwAoY1aHR0cDovL2NhY2VydHMuZGlnaWNlcnQuY29tL0RpZ2lDZXJ0VHJ1
# c3RlZFJvb3RHNC5jcnQwQwYDVR0fBDwwOjA4oDagNIYyaHR0cDovL2NybDMuZGln
# aWNlcnQuY29tL0RpZ2lDZXJ0VHJ1c3RlZFJvb3RHNC5jcmwwIAYDVR0gBBkwFzAI
# BgZngQwBBAIwCwYJYIZIAYb9bAcBMA0GCSqGSIb3DQEBCwUAA4ICAQB9WY7Ak7Zv
# mKlEIgF+ZtbYIULhsBguEE0TzzBTzr8Y+8dQXeJLKftwig2qKWn8acHPHQfpPmDI
# 2AvlXFvXbYf6hCAlNDFnzbYSlm/EUExiHQwIgqgWvalWzxVzjQEiJc6VaT9Hd/ty
# dBTX/6tPiix6q4XNQ1/tYLaqT5Fmniye4Iqs5f2MvGQmh2ySvZ180HAKfO+ovHVP
# ulr3qRCyXen/KFSJ8NWKcXZl2szwcqMj+sAngkSumScbqyQeJsG33irr9p6xeZmB
# o1aGqwpFyd/EjaDnmPv7pp1yr8THwcFqcdnGE4AJxLafzYeHJLtPo0m5d2aR8XKc
# 6UsCUqc3fpNTrDsdCEkPlM05et3/JWOZJyw9P2un8WbDQc1PtkCbISFA0LcTJM3c
# HXg65J6t5TRxktcma+Q4c6umAU+9Pzt4rUyt+8SVe+0KXzM5h0F4ejjpnOHdI/0d
# KNPH+ejxmF/7K9h+8kaddSweJywm228Vex4Ziza4k9Tm8heZWcpw8De/mADfIBZP
# J/tgZxahZrrdVcA6KYawmKAr7ZVBtzrVFZgxtGIJDwq9gdkT/r+k0fNX2bwE+oLe
# Mt8EifAAzV3C+dAjfwAL5HYCJtnwZXZCpimHCUcr5n8apIUP/JiW9lVUKx+A+sDy
# Divl1vupL0QVSucTDh3bNzgaoSv27dZ8/DCCBrAwggSYoAMCAQICEAitQLJg0pxM
# n17Nqb2TrtkwDQYJKoZIhvcNAQEMBQAwYjELMAkGA1UEBhMCVVMxFTATBgNVBAoT
# DERpZ2lDZXJ0IEluYzEZMBcGA1UECxMQd3d3LmRpZ2ljZXJ0LmNvbTEhMB8GA1UE
# AxMYRGlnaUNlcnQgVHJ1c3RlZCBSb290IEc0MB4XDTIxMDQyOTAwMDAwMFoXDTM2
# MDQyODIzNTk1OVowaTELMAkGA1UEBhMCVVMxFzAVBgNVBAoTDkRpZ2lDZXJ0LCBJ
# bmMuMUEwPwYDVQQDEzhEaWdpQ2VydCBUcnVzdGVkIEc0IENvZGUgU2lnbmluZyBS
# U0E0MDk2IFNIQTM4NCAyMDIxIENBMTCCAiIwDQYJKoZIhvcNAQEBBQADggIPADCC
# AgoCggIBANW0L0LQKK14t13VOVkbsYhC9TOM6z2Bl3DFu8SFJjCfpI5o2Fz16zQk
# B+FLT9N4Q/QX1x7a+dLVZxpSTw6hV/yImcGRzIEDPk1wJGSzjeIIfTR9TIBXEmtD
# mpnyxTsf8u/LR1oTpkyzASAl8xDTi7L7CPCK4J0JwGWn+piASTWHPVEZ6JAheEUu
# oZ8s4RjCGszF7pNJcEIyj/vG6hzzZWiRok1MghFIUmjeEL0UV13oGBNlxX+yT4Us
# SKRWhDXW+S6cqgAV0Tf+GgaUwnzI6hsy5srC9KejAw50pa85tqtgEuPo1rn3MeHc
# reQYoNjBI0dHs6EPbqOrbZgGgxu3amct0r1EGpIQgY+wOwnXx5syWsL/amBUi0nB
# k+3htFzgb+sm+YzVsvk4EObqzpH1vtP7b5NhNFy8k0UogzYqZihfsHPOiyYlBrKD
# 1Fz2FRlM7WLgXjPy6OjsCqewAyuRsjZ5vvetCB51pmXMu+NIUPN3kRr+21CiRshh
# WJj1fAIWPIMorTmG7NS3DVPQ+EfmdTCN7DCTdhSmW0tddGFNPxKRdt6/WMtyEClB
# 8NXFbSZ2aBFBE1ia3CYrAfSJTVnbeM+BSj5AR1/JgVBzhRAjIVlgimRUwcwhGug4
# GXxmHM14OEUwmU//Y09Mu6oNCFNBfFg9R7P6tuyMMgkCzGw8DFYRAgMBAAGjggFZ
# MIIBVTASBgNVHRMBAf8ECDAGAQH/AgEAMB0GA1UdDgQWBBRoN+Drtjv4XxGG+/5h
# ewiIZfROQjAfBgNVHSMEGDAWgBTs1+OC0nFdZEzfLmc/57qYrhwPTzAOBgNVHQ8B
# Af8EBAMCAYYwEwYDVR0lBAwwCgYIKwYBBQUHAwMwdwYIKwYBBQUHAQEEazBpMCQG
# CCsGAQUFBzABhhhodHRwOi8vb2NzcC5kaWdpY2VydC5jb20wQQYIKwYBBQUHMAKG
# NWh0dHA6Ly9jYWNlcnRzLmRpZ2ljZXJ0LmNvbS9EaWdpQ2VydFRydXN0ZWRSb290
# RzQuY3J0MEMGA1UdHwQ8MDowOKA2oDSGMmh0dHA6Ly9jcmwzLmRpZ2ljZXJ0LmNv
# bS9EaWdpQ2VydFRydXN0ZWRSb290RzQuY3JsMBwGA1UdIAQVMBMwBwYFZ4EMAQMw
# CAYGZ4EMAQQBMA0GCSqGSIb3DQEBDAUAA4ICAQA6I0Q9jQh27o+8OpnTVuACGqX4
# SDTzLLbmdGb3lHKxAMqvbDAnExKekESfS/2eo3wm1Te8Ol1IbZXVP0n0J7sWgUVQ
# /Zy9toXgdn43ccsi91qqkM/1k2rj6yDR1VB5iJqKisG2vaFIGH7c2IAaERkYzWGZ
# gVb2yeN258TkG19D+D6U/3Y5PZ7Umc9K3SjrXyahlVhI1Rr+1yc//ZDRdobdHLBg
# XPMNqO7giaG9OeE4Ttpuuzad++UhU1rDyulq8aI+20O4M8hPOBSSmfXdzlRt2V0C
# FB9AM3wD4pWywiF1c1LLRtjENByipUuNzW92NyyFPxrOJukYvpAHsEN/lYgggnDw
# zMrv/Sk1XB+JOFX3N4qLCaHLC+kxGv8uGVw5ceG+nKcKBtYmZ7eS5k5f3nqsSc8u
# pHSSrds8pJyGH+PBVhsrI/+PteqIe3Br5qC6/To/RabE6BaRUotBwEiES5ZNq0RA
# 443wFSjO7fEYVgcqLxDEDAhkPDOPriiMPMuPiAsNvzv0zh57ju+168u38HcT5uco
# P6wSrqUvImxB+YJcFWbMbA7KxYbD9iYzDAdLoNMHAmpqQDBISzSoUSC7rRuFCOJZ
# DW3KBVAr6kocnqX9oKcfBnTn8tZSkP2vhUgh+Vc7tJwD7YZF9LRhbr9o4iZghurI
# r6n+lB3nYxs6hlZ4TjCCBsIwggSqoAMCAQICEAVEr/OUnQg5pr/bP1/lYRYwDQYJ
# KoZIhvcNAQELBQAwYzELMAkGA1UEBhMCVVMxFzAVBgNVBAoTDkRpZ2lDZXJ0LCBJ
# bmMuMTswOQYDVQQDEzJEaWdpQ2VydCBUcnVzdGVkIEc0IFJTQTQwOTYgU0hBMjU2
# IFRpbWVTdGFtcGluZyBDQTAeFw0yMzA3MTQwMDAwMDBaFw0zNDEwMTMyMzU5NTla
# MEgxCzAJBgNVBAYTAlVTMRcwFQYDVQQKEw5EaWdpQ2VydCwgSW5jLjEgMB4GA1UE
# AxMXRGlnaUNlcnQgVGltZXN0YW1wIDIwMjMwggIiMA0GCSqGSIb3DQEBAQUAA4IC
# DwAwggIKAoICAQCjU0WHHYOOW6w+VLMj4M+f1+XS512hDgncL0ijl3o7Kpxn3GIV
# WMGpkxGnzaqyat0QKYoeYmNp01icNXG/OpfrlFCPHCDqx5o7L5Zm42nnaf5bw9Yr
# IBzBl5S0pVCB8s/LB6YwaMqDQtr8fwkklKSCGtpqutg7yl3eGRiF+0XqDWFsnf5x
# XsQGmjzwxS55DxtmUuPI1j5f2kPThPXQx/ZILV5FdZZ1/t0QoRuDwbjmUpW1R9d4
# KTlr4HhZl+NEK0rVlc7vCBfqgmRN/yPjyobutKQhZHDr1eWg2mOzLukF7qr2JPUd
# vJscsrdf3/Dudn0xmWVHVZ1KJC+sK5e+n+T9e3M+Mu5SNPvUu+vUoCw0m+PebmQZ
# BzcBkQ8ctVHNqkxmg4hoYru8QRt4GW3k2Q/gWEH72LEs4VGvtK0VBhTqYggT02ke
# fGRNnQ/fztFejKqrUBXJs8q818Q7aESjpTtC/XN97t0K/3k0EH6mXApYTAA+hWl1
# x4Nk1nXNjxJ2VqUk+tfEayG66B80mC866msBsPf7Kobse1I4qZgJoXGybHGvPrhv
# ltXhEBP+YUcKjP7wtsfVx95sJPC/QoLKoHE9nJKTBLRpcCcNT7e1NtHJXwikcKPs
# CvERLmTgyyIryvEoEyFJUX4GZtM7vvrrkTjYUQfKlLfiUKHzOtOKg8tAewIDAQAB
# o4IBizCCAYcwDgYDVR0PAQH/BAQDAgeAMAwGA1UdEwEB/wQCMAAwFgYDVR0lAQH/
# BAwwCgYIKwYBBQUHAwgwIAYDVR0gBBkwFzAIBgZngQwBBAIwCwYJYIZIAYb9bAcB
# MB8GA1UdIwQYMBaAFLoW2W1NhS9zKXaaL3WMaiCPnshvMB0GA1UdDgQWBBSltu8T
# 5+/N0GSh1VapZTGj3tXjSTBaBgNVHR8EUzBRME+gTaBLhklodHRwOi8vY3JsMy5k
# aWdpY2VydC5jb20vRGlnaUNlcnRUcnVzdGVkRzRSU0E0MDk2U0hBMjU2VGltZVN0
# YW1waW5nQ0EuY3JsMIGQBggrBgEFBQcBAQSBgzCBgDAkBggrBgEFBQcwAYYYaHR0
# cDovL29jc3AuZGlnaWNlcnQuY29tMFgGCCsGAQUFBzAChkxodHRwOi8vY2FjZXJ0
# cy5kaWdpY2VydC5jb20vRGlnaUNlcnRUcnVzdGVkRzRSU0E0MDk2U0hBMjU2VGlt
# ZVN0YW1waW5nQ0EuY3J0MA0GCSqGSIb3DQEBCwUAA4ICAQCBGtbeoKm1mBe8cI1P
# ijxonNgl/8ss5M3qXSKS7IwiAqm4z4Co2efjxe0mgopxLxjdTrbebNfhYJwr7e09
# SI64a7p8Xb3CYTdoSXej65CqEtcnhfOOHpLawkA4n13IoC4leCWdKgV6hCmYtld5
# j9smViuw86e9NwzYmHZPVrlSwradOKmB521BXIxp0bkrxMZ7z5z6eOKTGnaiaXXT
# UOREEr4gDZ6pRND45Ul3CFohxbTPmJUaVLq5vMFpGbrPFvKDNzRusEEm3d5al08z
# jdSNd311RaGlWCZqA0Xe2VC1UIyvVr1MxeFGxSjTredDAHDezJieGYkD6tSRN+9N
# UvPJYCHEVkft2hFLjDLDiOZY4rbbPvlfsELWj+MXkdGqwFXjhr+sJyxB0JozSqg2
# 1Llyln6XeThIX8rC3D0y33XWNmdaifj2p8flTzU8AL2+nCpseQHc2kTmOt44Owde
# OVj0fHMxVaCAEcsUDH6uvP6k63llqmjWIso765qCNVcoFstp8jKastLYOrixRoZr
# uhf9xHdsFWyuq69zOuhJRrfVf8y2OMDY7Bz1tqG4QyzfTkx9HmhwwHcK1ALgXGC7
# KP845VJa1qwXIiNO9OzTF/tQa/8Hdx9xl0RBybhG02wyfFgvZ0dl5Rtztpn5aywG
# Ru9BHvDwX+Db2a2QgESvgBBBijCCB4AwggVooAMCAQICEAiU+2IWwfbcTRWEwH9i
# ik0wDQYJKoZIhvcNAQELBQAwaTELMAkGA1UEBhMCVVMxFzAVBgNVBAoTDkRpZ2lD
# ZXJ0LCBJbmMuMUEwPwYDVQQDEzhEaWdpQ2VydCBUcnVzdGVkIEc0IENvZGUgU2ln
# bmluZyBSU0E0MDk2IFNIQTM4NCAyMDIxIENBMTAeFw0yMjA2MDMwMDAwMDBaFw0y
# NTA2MDMyMzU5NTlaMIGEMQswCQYDVQQGEwJBVTEYMBYGA1UECBMPU291dGggQXVz
# dHJhbGlhMREwDwYDVQQHEwhBZGVsYWlkZTEjMCEGA1UEChMaQ2xpY2sgU3R1ZGlv
# cyAoU0EpIFB0eSBMdGQxIzAhBgNVBAMTGkNsaWNrIFN0dWRpb3MgKFNBKSBQdHkg
# THRkMIICIjANBgkqhkiG9w0BAQEFAAOCAg8AMIICCgKCAgEAwVvn4P1gVJX4+7PB
# Gl7y0EWV2nFOFcYQSmT6ThhzbRh1aq3DbKLuca+a6mA6JfnnbhX6tlxv1kbXdmbj
# mxlwYbyIVE+HIcBxWlXf62KrEzhe1yLLlO3kSFJezbtHt4kgMxHJsbXrs4AI8k2W
# 428nqNZoPkfX52dpuxkBdmLdLdIO1VwfW+5Eq3asKJ9O6Sxz4WcQv7Cq/mwbFY9B
# b5DygoKYHjY04Dg1ODH091LEhGaz9YEM+BaoKgiUhFzsy8KxNnno3VMFQe6sY8nm
# Z9Ty5lMVBabBPIuu6hS6kZXlTKiFcwiHBKor7Fs8lGTtqq+IBq57AxsGrmRZ9zjR
# sr6EmMNP/CXopho/Cq6l9+E4rgnUb5auZbTbqOXASDINKTPzLI80MA1JXkMbX/NW
# fZ91RAvf/0LCENzjBr7IhxL1YyoC9ktRS0EqGiMiz+fLJ/4ZMT+z/a1TFccBPq9m
# aRORMIouuvI1o0dk8C20+lOiCCJLiuI8lRxqK7QKftbtRYmedoE7AIKMOk0IsCOc
# d9gSjLFCNXAVagBKSJQj2KNvX/5Vd2y3D/2cKco4rjZuMdigJ2jpL1Cv94A0V88+
# 3NZr1axb03Q5ZFpcYzm7oGqTk6LwDZ9sENWCV2gUGvaMOccnNCSAdOfPPNESAVf4
# mnG/e1NYniQSh0pjp7cMmw/xyMECAwEAAaOCAgYwggICMB8GA1UdIwQYMBaAFGg3
# 4Ou2O/hfEYb7/mF7CIhl9E5CMB0GA1UdDgQWBBT2B+2xqYAg0+Mwh5ESJ60t4URN
# tDAOBgNVHQ8BAf8EBAMCB4AwEwYDVR0lBAwwCgYIKwYBBQUHAwMwgbUGA1UdHwSB
# rTCBqjBToFGgT4ZNaHR0cDovL2NybDMuZGlnaWNlcnQuY29tL0RpZ2lDZXJ0VHJ1
# c3RlZEc0Q29kZVNpZ25pbmdSU0E0MDk2U0hBMzg0MjAyMUNBMS5jcmwwU6BRoE+G
# TWh0dHA6Ly9jcmw0LmRpZ2ljZXJ0LmNvbS9EaWdpQ2VydFRydXN0ZWRHNENvZGVT
# aWduaW5nUlNBNDA5NlNIQTM4NDIwMjFDQTEuY3JsMD4GA1UdIAQ3MDUwMwYGZ4EM
# AQQBMCkwJwYIKwYBBQUHAgEWG2h0dHA6Ly93d3cuZGlnaWNlcnQuY29tL0NQUzCB
# lAYIKwYBBQUHAQEEgYcwgYQwJAYIKwYBBQUHMAGGGGh0dHA6Ly9vY3NwLmRpZ2lj
# ZXJ0LmNvbTBcBggrBgEFBQcwAoZQaHR0cDovL2NhY2VydHMuZGlnaWNlcnQuY29t
# L0RpZ2lDZXJ0VHJ1c3RlZEc0Q29kZVNpZ25pbmdSU0E0MDk2U0hBMzg0MjAyMUNB
# MS5jcnQwDAYDVR0TAQH/BAIwADANBgkqhkiG9w0BAQsFAAOCAgEAHhdX15IL1zt7
# TUgYpDJXht7qdsO7dhtHLvfcXMhNZ2WR6k4P4Fy0M531ztmc4HXcjasd0YbepR2x
# RrpdGWERhe2Rbb8K2HmrUhQjwcrsgNuk3vdA1I1ROF09i3DjwBZahDWq7uwAXM5Y
# OKdFy8qGf5cs6bjhpyI6JcM9qgE92s22FwXmoc/8eI/Ux7kZfwD2fXxWs/2kobEH
# /YP3VzLeNgrORegxlBcczIWsYs/1+aT9E7MI+v+aJ+8/f9FuASM+ez45guH+RDRD
# hzt+h/+ADOTg4g7vMERoaaUnuhWVJB6gxYeYb4bH/UGhLgFhWY9wajeOHREqE3Yw
# 9MLsZqsGrPSP8bjX47+672Te4fr/L6iX5gwsZj03SNrsY/LKHwfSR+71Fq+mAwTv
# OUA2hfc8AD60Bc79Y2T3JenVf97JL+AKrRZN4sMCOOlax5JNgZ2+niMWkrjHE5R9
# bg4ssWchLn2SSJiZzHR969Umm8OoM+UGjopdNQsP0JX3jrNzrP/qkONLOOo70EFx
# /sKcbE0ii+a3NfPav3I62ScRqoSdEcCHSSRMnLzFoyACaG+Oy0+LKBKHnND9PX9Z
# g8fc42syrwUeVfEhZebWRr+wPw7BZExfYDVJUEbCkb5Az4nBt2ZeS28yi25fuljx
# gTJr0mjxiY6/1hyZIe1tR6q8IXfJOvkxggZCMIIGPgIBATB9MGkxCzAJBgNVBAYT
# AlVTMRcwFQYDVQQKEw5EaWdpQ2VydCwgSW5jLjFBMD8GA1UEAxM4RGlnaUNlcnQg
# VHJ1c3RlZCBHNCBDb2RlIFNpZ25pbmcgUlNBNDA5NiBTSEEzODQgMjAyMSBDQTEC
# EAiU+2IWwfbcTRWEwH9iik0wCQYFKw4DAhoFAKB4MBgGCisGAQQBgjcCAQwxCjAI
# oAKAAKECgAAwGQYJKoZIhvcNAQkDMQwGCisGAQQBgjcCAQQwHAYKKwYBBAGCNwIB
# CzEOMAwGCisGAQQBgjcCARUwIwYJKoZIhvcNAQkEMRYEFCnHE+TG5eVxbFX6i907
# 7objwxhXMA0GCSqGSIb3DQEBAQUABIICAHNRpRCkg4I1K/cpDSzfXzbO2BpL8mxn
# cFvjsK8QwKT1yUCzQNJvxAYdHfe5QTjkQ0tkJbEh4NlbzwKtcxkE31MPxZuRRsQj
# AAtDcgfPW5WRYKxDNuIgHcenDm546Rj7GZMZN7hqkuxzSlFWj3i3RFVHRCKmcsJy
# 0B7Mwn5a+bxPM1cwRgYPmkvUapLNFUDCj8o2ZxGFt8Q42fpB9mBqwCnjAUHfpqJX
# JFSZm7lrkA2dOG3T2JMhuJLDbNXy9Ud8QOUJVsuKkSeD5atT1bg3prJFbNjd1azP
# 9f8xqV8txzMdHr57m+a8Zx+I+H1I4KfVXk8G1qWcMeHjMMypzwAKBWQCBoalAOuo
# 4ezN3Av30AGv/clMZHR0MITDWNumzTO8kEGFqQrB6Bjom2UlMHThzDBj/Ve0DuM2
# Ho30A5sJVvwk4HeQdmiO2zVJVOcLBwTGgDRg4oqKwAgkXiSE1vCkWOChCfUxxTNz
# dh0EH5fcHpMQ+QrFcbZ6gpcvgWjwrFT+nHlWVeL+fR3meYkIcw1uI8laZYkjVvVj
# IlUIIMERWOgmL82+uUVsPWFkXT3Ar612Ksqv/5M9zFmtwDkYsBAoycs0jWe3drTA
# k4gP9Ol0vAMAjOvnOxrujC1IDFSG9inrONQ8+sKSlJ7Tb5NZz/wROeVfQxkHaPLi
# 5p0OEFICgBhvoYIDIDCCAxwGCSqGSIb3DQEJBjGCAw0wggMJAgEBMHcwYzELMAkG
# A1UEBhMCVVMxFzAVBgNVBAoTDkRpZ2lDZXJ0LCBJbmMuMTswOQYDVQQDEzJEaWdp
# Q2VydCBUcnVzdGVkIEc0IFJTQTQwOTYgU0hBMjU2IFRpbWVTdGFtcGluZyBDQQIQ
# BUSv85SdCDmmv9s/X+VhFjANBglghkgBZQMEAgEFAKBpMBgGCSqGSIb3DQEJAzEL
# BgkqhkiG9w0BBwEwHAYJKoZIhvcNAQkFMQ8XDTI0MDMxMzIxMzAyMlowLwYJKoZI
# hvcNAQkEMSIEILpll27lKd+XQffCoelZIiTgCopp4e5ly4nwGCbee5J4MA0GCSqG
# SIb3DQEBAQUABIICAIfLH7VilQTqna69f1SAetyLJMUDX2yXFb+ojYt10r2NbBCd
# 0HzmWuI2WoY3xuFu8vQYEmPw8aLSNTRZiCldTh1dvrEjyIFs6NVpzJsDmnonblUl
# 3SDG9TBPbSfZZLbkoreXNLfOGXlNups8S2a0ZOJFjTH3EK4/8IiLfn+aLZHA0Ju2
# 3DeEIcGEyQb7MTrDp0xhiaZS1+OvLSiAy3lt/K1YsZmRCDgge1h6vRhdWxQIAtHs
# uZ0QAB/TPl8bKrfrTwd2XoxdOqK6zdYvq4Nkf5CyO/+8diO86jO/mJn8Gq+Y/S/e
# AquoRJ2NSdFKJgJFm6EMfPWF8x87giFJ8ciQJPfms5PIUu9Z9ZFUPn2tFG6Kwt6b
# DQcGLNypwIZ9fVB8GHAdnBv1uPSBhK2aPdIzarzHixEzD1slMFFAAvNHwjyHTfny
# PqIK9Ov64QTEF2IPlgz1ph42lYVdKxANzyC9MGkj9sF1TnFU8WIQCWoaZnBC/41r
# Wgc8eVTwarwkvjPhhwLNCW6M+EqDPdW8POXQPNIZ+dOy3jDT9sGf3Owsmw6QrxVo
# dFyhWo3ehZdzPirA7NSx5ueisfPan+71Ei6SN2bGdPgy2aIuQ+uvKKm0JjvoMa7+
# zpE58K6H+NrE/LcUpHqgGIR1MYL4FI+SoPQQtcXdzQBadpYB41I16q9yg0Vy
# SIG # End signature block
